"""
#    TOP2049 Open Source programming suite
#
#    Winbond W29EE011 DIP32
#    Winbond W29EE011 PLCC32 (inside 1:1 PLCC32->DIP32 adapter)
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_w29ee011dip32(Chip):
	PROGCMD_WRITEBUF	= 1 # Write buffer to chip

	STAT_BUSY		= 0x01 # Programmer is running a command

	def __init__(self):
		Chip.__init__(self,
			      chipPackage = "DIP32",
			      chipPinVCC = 32,
			      chipPinsVPP = None,
			      chipPinGND = 16)

	def erase(self):
		self.applyVCC(True)
		self.applyVPP(True)
		self.applyGND(True)

		commands = (
			(0x5555, 0xAA),
			(0x2AAA, 0x55),
			(0x5555, 0x80),
			(0x5555, 0xAA),
			(0x2AAA, 0x55),
			(0x5555, 0x10),
		)

		self.progressMeterInit("Erasing chip", 0)
		self.__setCEOE(CE=0, OE=1)

		self.__resetBufferPointers()
		self.__swDataProtect(False)
		self.__runCommandSync(self.PROGCMD_WRITEBUF)
		self.top.hostDelay(0.05)
		self.__resetBufferPointers()
		for command in commands:
			self.__appendJEDEC(command[0], command[1])
		self.__runCommandSync(self.PROGCMD_WRITEBUF)
		self.top.hostDelay(0.05)

		self.__setCEOE(CE=1, OE=1)
		self.progressMeterFinish()

	def readEEPROM(self):
		self.applyVCC(True)
		self.applyVPP(True)
		self.applyGND(True)

		self.progressMeterInit("Reading EEPROM", 0x20000)
		self.__setCEOE(CE=0, OE=0)
		image = self.__readRange(0, 0x20000, progress=True)
		self.__setCEOE(CE=1, OE=1)
		self.progressMeterFinish()

		return image

	def __readRange(self, baseAddress, size, progress=False):
		image = ""
		byteCount = 0
		prevAddr = baseAddress
		self.__loadReadAddrLo(baseAddress)
		self.__loadReadAddrMed(baseAddress >> 8)
		self.__loadReadAddrHi(baseAddress >> 16)
		for offset in range(0, size):
			addr = baseAddress + offset
			if progress:
				self.progressMeter(addr)
			if (addr & 0xFF) != (prevAddr & 0xFF):
				self.__loadReadAddrLo(addr)
			if (addr & 0xFF00) != (prevAddr & 0xFF00):
				self.__loadReadAddrMed(addr >> 8)
			if (addr & 0xFF0000) != (prevAddr & 0xFF0000):
				self.__loadReadAddrHi(addr >> 16)
			prevAddr = addr
			self.top.cmdFPGARead(0x10)
			byteCount += 1
			if byteCount == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(byteCount)
				byteCount = 0
		image += self.top.cmdReadBufferReg(byteCount)
		return image

	def writeEEPROM(self, image):
		if len(image) > 0x20000:
			self.throwError("Invalid EPROM image size %d (expected <=%d)" %\
				(len(image), 0x20000))

		self.applyVCC(True)
		self.applyVPP(True)
		self.applyGND(True)

		self.progressMeterInit("Writing EEPROM", len(image))
		self.__setCEOE(CE=0, OE=1)
		for addr in range(0, len(image), 128):
			self.progressMeter(addr)
			pagelen = min(128, len(image) - addr)
			page = image[addr:addr+pagelen]
			self.__writePage(addr, page)
		self.__setCEOE(CE=1, OE=1)
		self.progressMeterFinish()

	def __swDataProtect(self, enable):
		if enable:
			jedecCommands = (
				(0x5555, 0xAA),
				(0x2AAA, 0x55),
				(0x5555, 0xA0),
			)
		else:
			jedecCommands = (
				(0x5555, 0xAA),
				(0x2AAA, 0x55),
				(0x5555, 0x80),
				(0x5555, 0xAA),
				(0x2AAA, 0x55),
				(0x5555, 0x20),
			)
		for command in jedecCommands:
			self.__appendJEDEC(command[0], command[1])

	def __writePage(self, pageAddress, pageData):
		for t in range(0, 15):
			self.__resetBufferPointers()
			self.__swDataProtect(True)
			assert(len(pageData) <= 128)
			for byte in pageData:
				self.__writeBufAppend(byte2int(byte))
			self.__loadWriteAddr(pageAddress)
			self.__runCommandSync(self.PROGCMD_WRITEBUF)
			self.top.hostDelay(0.01)
			# Verify
			self.__setCEOE(CE=0, OE=0)
			verifyImage = self.__readRange(pageAddress, len(pageData))
			self.__setCEOE(CE=0, OE=1)
			if verifyImage == pageData:
				break
			self.top.hostDelay(0.1)
		else:
			self.throwError("Verify error on page write at address 0x%05X" % pageAddress)

	def __writeBufAppend(self, byte):
		# This also auto-increments the write buffer pointer
		self.top.cmdFPGAWrite(0x10, byte & 0xFF)

	def __resetBufferPointers(self):
		self.top.cmdFPGAWrite(0x13, 0)

	def __loadCommand(self, command):
		self.top.cmdFPGAWrite(0x12, command & 0xFF)

	def __runCommandSync(self, command):
		self.__loadCommand(command)
		self.__busyWait()

	def __loadWriteAddr(self, addr):
		self.__loadWriteAddrLo(addr)
		self.__loadWriteAddrMed(addr >> 8)
		self.__loadWriteAddrHi(addr >> 16)

	def __loadWriteAddrLo(self, addrLo):
		self.top.cmdFPGAWrite(0x14, addrLo & 0xFF)

	def __loadWriteAddrMed(self, addrMed):
		self.top.cmdFPGAWrite(0x15, addrMed & 0xFF)

	def __loadWriteAddrHi(self, addrHi):
		self.top.cmdFPGAWrite(0x16, addrHi & 0xFF)

	def __loadReadAddrLo(self, addrLo):
		self.top.cmdFPGAWrite(0x17, addrLo & 0xFF)

	def __loadReadAddrMed(self, addrMed):
		self.top.cmdFPGAWrite(0x18, addrMed & 0xFF)

	def __loadReadAddrHi(self, addrHi):
		self.top.cmdFPGAWrite(0x19, addrHi & 0xFF)

	def __setCEOE(self, CE, OE):
		data = 0
		if CE:
			data |= 0x01
		if OE:
			data |= 0x02
		self.top.cmdFPGAWrite(0x1A, data)

	def __appendJEDEC(self, addr, data):
		self.__loadJEDECAddrLo(addr)
		self.__loadJEDECAddrMed(addr >> 8)
		self.__loadJEDECAddrHi(addr >> 16)
		self.__loadJEDECData(data)

	def __loadJEDECAddrLo(self, addrLo):
		self.top.cmdFPGAWrite(0x1B, addrLo & 0xFF)

	def __loadJEDECAddrMed(self, addrMed):
		self.top.cmdFPGAWrite(0x1C, addrMed & 0xFF)

	def __loadJEDECAddrHi(self, addrHi):
		self.top.cmdFPGAWrite(0x1D, addrHi & 0xFF)

	def __loadJEDECData(self, data):
		# This also auto-increments the JEDEC buffer pointer
		self.top.cmdFPGAWrite(0x1E, data & 0xFF)

	def __getStatusFlags(self):
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def __busy(self):
		return bool(self.__getStatusFlags() & self.STAT_BUSY)

	def __busyWait(self):
		for i in range(0, 100):
			if not self.__busy():
				return
			self.top.hostDelay(0.01)
		self.throwError("Timeout in busywait.")

ChipDescription(
	Chip_w29ee011dip32,
	bitfile = "w29ee011dip32",
	runtimeID = (0x0009, 0x01),
	chipType = ChipDescription.TYPE_EEPROM,
	chipVendors = "Winbond",
	description = "W29EE011 EEPROM",
	packages = ( ("DIP32", ""), ("PLCC32", "Use 1:1 PLCC32->DIP32 adapter"), ),
	broken = True,
)



"""
#    TOP2049 Open Source programming suite
#
#    HM62256 DIP28 SRAM support
#
#    Copyright (c) 2011 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from generic_sram import *


class Chip_HM62256DIP28(Chip_genericSRAM):
	def __init__(self):
		Chip_genericSRAM.__init__(self,
			chipPackage = "DIP28",
			chipPinVCC = 28,
			chipPinGND = 14,
			VCCVoltage = 5,
			nrAddressBits = 15,
			nrDataBits = 8,
		)

ChipDescription(
	Chip_HM62256DIP28,
	bitfile = "hm62256dip28",
	runtimeID = (0x000A, 0x01),
	chipType = ChipDescription.TYPE_SRAM,
	chipVendors = "S@Tech",
	description = "HM62256 SRAM",
	packages = ( ("DIP28", ""), )
)



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8 common - basic file for 16bit PIC MCU
#
#    Copyright (c) 2012 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *

class Chip_Microchip16_common(Chip):
	
	STAT_BUSY = 0x01
	STAT_SDIO = 0x02

	PROGCMD_SENDSIXINSTR	 = 0
	PROGCMD_SENDREGOUTINSTR	 = 1
	PROGCMD_ENTERPM 	 = 2
	PROGCMD_SEND9SIXINSTR	 = 3
	
	codeExitResetVector = (0x000000, 0x040200, 0x000000)
	codeExitResetVectorSimple = (0x040200, 0x000000)
	codeInitializeW7toVISI = (0x207847, 0x000000)
	codeResetDeviceInternalPC = (0x040200, 0x000000)

	# EEPROM access: default on, if does not exist override it
	hasEEPROM = True
	eepromStartAddress = 0x7FFE00
	
	# default delays - can be overridden
	delayTdis = 0.0001
	delayTprog = 0.001
	delayCommandDataREGOUT = 0.0000005
	delayTera = 0.01
	# delayP3 = 0.000000015 # Input Data Hold Time from PGECx rise
	delayP3 = 0.00000006  # this is hard-tuned value !
	delayP4 = 0.000000040  # Delay between 4-Bit Command Operand and Command Operand
	delayP4A = 0.000000040  # Delay between 4-Bit Command Operand and the Next 4-bit command
	delayP5 = 0.000000020  # Delay between Last PGCx fall of Command Byte and First PGC rise by Programming Executive
	delayP6 = 0.000000100  # Vdd rise setup time to nMCLR rise
	delayP7 = 0.025  # Input data Hold Time from nMCLR rise, Vpp fall
	delayP11 = 0.0025  # chip Erase Time
	delayP13 = 0.00125  # row programming time
	delayP16 = 0  # delay between last PGCx fall and nMCLR fall
	delayP17 = 0.000000100  # nMCLR fall to Vdd fall
	delayP18 = 0.001  # delay between first nMCLR fall and first PGCx rise
	delayP19 = 0.001  # delay between last PGC fall for key sequence on PGDx and second nMCLR rise
  
  	deviceIDAddr = 0xFF0000
	deviceIDLength = 1
	
	deviceIdMapDict = {
			0x0d00:"24f04ka201", 0x0d02:"24f04ka200", 0x4b14:"24f16kl402", 0x4b1e:"24f16kl401", 0x4b04:"24f08kl402", 0x4b0e:"24f08kl401",
			0x4b00:"24f08kl302", 0x4b0a:"24f08kl301", 0x4b06:"24f08kl201", 0x4b05:"24f08kl200", 0x4b02:"24f04kl101", 0x4b01:"24f04kl100"
	}
	def getIHexInterpreter(self):
		inter = IHexInterpreter()
		inter.progmemRanges = [ AddressRange(0, 2 * self.flashPageSize) ]
		inter.fuseRanges = [ AddressRange(2 * self.configWordAddr,
						  2 * self.configWordAddr + 1) ]
		return inter
		
	@classmethod
	def getSupportFlags(cls):
		flags = super(Chip_Microchip16_common, cls).getSupportFlags()
		if not cls.hasEEPROM:
			flags &= ~(Chip.SUPPORT_EEPROMREAD | \
				   Chip.SUPPORT_EEPROMWRITE)
		return flags
	
	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip.__init__(self,
		chipPackage=chipPackage,
		chipPinVCC=chipPinVCC,
		chipPinsVPP=chipPinsVPP,
		chipPinGND=chipPinGND)
		self.signature = signature
		self.flashPageSize = flashPageSize  # Flash page size, in words
		self.flashPages = flashPages  # Nr of flash pages
		self.eepromPageSize = eepromPageSize  # EEPROM page size, in bytes
		self.eepromPages = eepromPages  # Nr of EEPROM pages
		self.fuseBytes = fuseBytes  # Nr of fuse bytes
		self.isInPmMode = False
		self.BufferedBytes = 0
		self.Image = ""
		
	def enterPM(self, lowVoltageIcspEntry=True):
		if self.isInPmMode:
			return
		"Enter HV programming mode. Vdd first entry mode"
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(False)
		self.setPins(0, 0)
		self.top.cmdSetVCCVoltage(self.voltageVDD)
		self.top.cmdSetVPPVoltage((self.voltageVPP, self.voltageVDD)[lowVoltageIcspEntry])
		self.applyGND(True)
		self.applyVCC(True)
		self.top.hostDelay(self.delayP6)
		self.applyVPP(True)
		self.setTopProgrammerDelays();
		if lowVoltageIcspEntry:
			self.top.hostDelay(self.delayP6)
			self.applyVPP(False)
		self.top.hostDelay(self.delayP18)
		self.sendCommand(self.PROGCMD_ENTERPM)
		if lowVoltageIcspEntry:
			self.top.hostDelay(self.delayP19)
			self.applyVPP(True)
		self.top.hostDelay(self.delayP7)
		self.sendSIX(0x0, True)
		self.executeCode(self.codeExitResetVectorSimple)
		# self.testPORTA()
		self.isInPmMode = True

	def checkSignature(self):
		signature = self.readSignature()
		if signature != self.signature:
			msg = "Unexpected device signature. " + \
			 "Want %02X%02X%02X, but got %02X%02X%02X" % \
			(byte2int(self.signature[0]), byte2int(self.signature[1]),
			byte2int(self.signature[2]),
			byte2int(signature[0]), byte2int(signature[1]),
			byte2int(signature[2]))
		if self.top.getForceLevel() >= 1:
			self.printWarning(msg)
		else:
			self.throwError(msg)
			
	def old_readSignature(self):
		self.progressMeterInit("Reading signature", 0)
		self.enterPM()
		self.Image = ""
		self.BufferedBytes = 0
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeInitializeTBLPAG(self.deviceIDAddr, 6))
		self.executeCode(self.codeInitializeW7toVISI)
		self.read2words()
		self.progressMeterFinish()
		return self.top.cmdReadBufferReg(4)
		
	def erase(self):
			self.__erase()
				
	def __erase(self, keepEEPROM=False):
		def er(NVMCON):
			self.executeCode(self.codeExitResetVector)
			# Set the NVMCON to erase entire PGM
			self.executeCode(self.getCodeSetNVMCON(NVMCON))
			# Set the TBLPAG and perform dummy table write to select the erased memory
			self.executeCode(self.getCodeInitializeTBLPAG(0, 0))
			self.executeCode((0xBB0800, 0x000000, 0x000000))
			# Initiate the erase cycle
			self.executeCode((0xA8E761, 0x000000, 0x000000))
			while self.isWRset():
				pass
		self.progressMeterInit("Erasing chip", 0)
		self.enterPM()
		er(0x4064)
		if(not keepEEPROM and self.eepromPageSize != 0):
			er(0x4050)
		self.progressMeterFinish()
		self.exitPM()

	def readProgmem(self):	
		def unpackImage():
			out = ""
			for halfPackAddr in range(0, len(self.Image), 6):
				out += self.Image[halfPackAddr:halfPackAddr + 3] + "\0"
				out += self.Image[halfPackAddr + 4:halfPackAddr + 6] + self.Image[halfPackAddr + 3] + "\0"
			return out
	
		nrWords = self.flashPages * self.flashPageSize
		# return self.readSequentialBlock(0x0 , nrWords/2, "Reading Progmem")
		# something wrong for packed PM reading below
		self.enterPM()
		self.progressMeterInit("Reading flash", nrWords / 2)
		self.BufferedBytes = 0
		self.Image = ""
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeInitializeTBLPAG(0, 6))
		self.executeCode(self.codeInitializeW7toVISI)
		for wordAddrHalf in range(0, nrWords / 2):
			self.executeCode((0xBA0B96, 0x0, 0x0))
			self.readREGOUTword()
			# Microchip's bug in the documentation
			# TBLRDH.B [W6++], [W7++] is correct instead of TBLRDH [W6++], [W7]
			self.executeCode((0x0, 0xBADBB6, 0x0, 0x0, 0xBAD3D6, 0x0, 0x0))
			self.readREGOUTword()
			self.executeCode((0x0, 0xBA0BB6, 0x0, 0x0))
			self.readREGOUTword()
			self.executeCode(self.codeExitResetVector)
			self.progressMeter(wordAddrHalf)
		self.progressMeterFinish()
		self.flushBufferToImage()
		self.exitPM()
		# return self.Image
		return unpackImage()
		

	def _t_readEEPROM(self):	
		nrWords = self.eepromPages * self.eepromPageSize
		self.enterPM()
		self.progressMeterInit("Reading EEPROM", nrWords)
		self.BufferedBytes = 0
		self.Image = ""
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeInitializeTBLPAG(0x7F0000 | 0, 6))
		self.executeCode((0x207847, 0x0))
		for wordAddr in range(0, nrWords):
			self.executeCode((0xBA0B96, 0x0, 0x0))
			self.readREGOUTword()
			self.executeCode(self.codeExitResetVector)
			self.progressMeter(wordAddr)
		self.progressMeterFinish()
		self.flushBufferToImage()
		return self.Image
	
	def readEEPROM(self):
		nrWords = self.eepromPages * self.eepromPageSize
		return self.readSequentialBlock(self.eepromStartAddress , nrWords, "Reading EEPROM")
	
	def readFuse(self):
		return self.readSequentialBlock(self.configWordAddr, self.fuseBytes / 2, "Reading Config Words")
	6402
	def tmp_readSignature(self):
		self.enterPM()
		self.executeCode(self.codeExitResetVector)
		self.executeCode((0x200FF0, 0x880190, 0x200006, 0x0, 0x0, 0x207847, 0x0, 0xBA0BB6, 0x0, 0x0, 0x0))
		self.Image = ""
		self.readREGOUTword()
		self.executeCode(self.codeExitResetVector)
		self.flushBufferToImage()
		signature = self.top.cmdReadBufferReg()[0:2]
		devId = (byte2int(signature[1]) << 8) | byte2int(signature[0])
		if(devId in self.deviceIdMapDict):
			print("device: {:s}".format(self.deviceIdMapDict.get(devId)))
		else:
			print("WARNING: device id {:o} not found in local dictionary".format(devId))
		return self.Image
	def readSignature(self):
		signature = self.readSequentialBlock(self.deviceIDAddr, self.deviceIDLength, "Reading Signature")
		devId = (byte2int(signature[1]) << 8) | byte2int(signature[0])
		if(devId in self.deviceIdMapDict):
			print("device: {:s}".format(self.deviceIdMapDict.get(devId)))
		else:
			print("WARNING: device id {:x} not found in local dictionary".format(devId))
		return signature
		
	def testPORTA(self):
		self.executeCode((0x0, 0x23CFF4, 0x881644, 0x0, 0xEB8200, 0x881654, 0x0, 0x0))
	def readSequentialBlock(self, startAddr, nWords, infoText):	
		self.enterPM()
		self.progressMeterInit(infoText, nWords)
		self.BufferedBytes = 0
		self.Image = ""
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeInitializeTBLPAG(startAddr, 6))
		self.executeCode(self.codeInitializeW7toVISI)		
		for wordAddr in range(0, nWords):
			# print "wAddr:{:x}".format(wordAddr)
			self.executeCode((0xBA0BB6, 0x0, 0x0))
			self.readREGOUTword()
			self.executeCode((0x0,))
			self.progressMeter(wordAddr)
		self.executeCode(self.codeExitResetVectorSimple)
		self.progressMeterFinish()
		self.flushBufferToImage()
		self.exitPM()
		return self.Image
				
	def writeProgmem(self, image):
		def writePackedInstructionWords(addr, _16bytes):
			"""
			writes 4 double word size instructions to program memory.
			Most high instruction byte has 0x00 value and thus is ignored.
			"""
			print("writingPackedInstr init, addr: {:x}\n".format(addr))
			self.executeCode(self.getCodeInitializeTBLPAG(addr, 7))
			cnt = 0
			for wIdx in (0, 8):
			       # LSW0
			       WD = (byte2int(_16bytes[wIdx + 1]) << 8) | byte2int(_16bytes[wIdx + 0])
			       self.executeCode(((0x200000 | (WD << 4) | cnt),))
			       cnt += 1
			       # MSB1 MSB0
			       WD = (byte2int(_16bytes[wIdx + 6]) << 8) | byte2int(_16bytes[wIdx + 2])
			       self.executeCode(((0x200000 | (WD << 4) | cnt),))
			       cnt += 1
			       # LSW1
			       WD = (byte2int(_16bytes[wIdx + 5]) << 8) | byte2int(_16bytes[wIdx + 4])
			       self.executeCode(((0x200000 | (WD << 4) | cnt),))
			       cnt += 1
			self.executeCode((0xEB0300, 0x0))
			print("writingPackedInstr med\n")
			# Set the Read Pointer and load the (next set of) write latches
			for i in range(0, 2):
				self.executeCode((0xBB0BB6, 0x0, 0x0, 0xBBDBB6, 0x0, 0x0, 0xBBEBB6, 0x0, 0x0, 0xBB1BB6, 0x0, 0x0))
			print("writingPackedInstr done\n")
		def writeSeq():
			# Initiate the write cycle
			self.executeCode((0xA8E761, 0x0, 0x0))
			while self.isWRset():
				pass
			self.executeCode((0x040200, 0x0))

		nrWords = self.flashPages * self.flashPageSize
		if len(image) > (nrWords * 4) or len(image) % 4 != 0:
			self.throwError("Invalid flash image size %d (expected <=%d and double word aligned)" % \
				(len(image), nrWords * 4))
		self.progressMeterInit("Writing flash", len(image) // 12)
		self.enterPM()
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeSetNVMCON(0x4004))
		for packAddr in range(0, len(image) // 16):
			self.progressMeter(packAddr)
			writePackedInstructionWords(packAddr * 8, image[16 * packAddr:][:16])
			if(0 == ((packAddr + 1) % 8)):
				writeSeq()
		writeSeq()
		self.progressMeterFinish()
		self.exitPM()
		
		
	def writeEEPROM(self, image):
		nrWords = self.eepromPages * self.eepromPageSize
		if len(image) > nrWords * 2:
			self.throwError("Invalid flash image size {:d} (expected <={:d})".format(len(image), 2 * nrWords))
		self.enterPM()
		self.progressMeterInit("Writing eeprom", len(image) / 2)
		self.executeCode(self.codeExitResetVector)
		self.executeCode(self.getCodeSetNVMCON(0x4004))
		for addr in range(0, len(image) / 2):
			self.progressMeter(addr)
			WD = (byte2int(image[addr * 2 + 1]) << 8) | byte2int(image[addr * 2 + 0])
			if WD != 0xFFFF:
				self.executeCode(self.getCodeInitializeTBLPAG(self.eepromStartAddress | (2 * addr), 7))
				# Load W0 with data word program and load the wire latch 
				self.executeCode((0x200000 | (WD << 4), 0xBB1B80, 0x0, 0x0))
				# Initiate the write cycle
				self.executeCode((0xA8E761, 0x0, 0x0))
				while self.isWRset():
					pass
				self.executeCode(self.codeExitResetVectorSimple)
				
				
		self.progressMeterFinish()
		self.exitPM()
		
	def writeFuse(self, image):
		if len(image) != self.fuseBytes:
			self.throwError("Invalid Fuses image size %d (expected %d)" % \
				(len(image), self.fuseBytes))
		self.progressMeterInit("Writing fuses", 0)
		# print "image1:%x,,%x,,%x" % (byte2int(image[0]),byte2int(image[1]),byte2int(image[1])<<8)
		CW = []
		for tBytes in zip(image[::2], image[1::2]):
			CW.append((byte2int(tBytes[1]) << 8) | byte2int(tBytes[0]))
		self.writeConfigWord(CW)
		self.progressMeterFinish()
    
	def writeConfigWord(self, listConfigWord16):
		self.enterPM()
		self.executeCode(self.codeExitResetVector)
		# init WritePointer (W7) for the TBLWT instruction, Set the NVMCON reg. to prog. Config. regs.
		self.executeCode((0x200007, 0x24004A, 0x883B0A))
		# init TBLPAG
		# another bug in Microchip's doc  - s/0x200F86/0x200F80/
		self.executeCode((0x200F80, 0x880190))
		for configWord16 in listConfigWord16:
			print ("write CW 0x{:x}".format(configWord16))
			# Load the Config reg data to W6
			self.executeCode(((0x200006 | (configWord16 << 4)),))
			# Write the Config. reg. data to the write latch and increment Write Pointer
			self.executeCode((0x0, 0xBB1B86, 0x0, 0x0))
			# Initiate the write cycle
			self.executeCode((0xA8E761, 0x0, 0x0))
			while self.isWRset():
				pass
			self.executeCode(self.codeExitResetVectorSimple)
		self.top.flushCommands()
		self.progressMeterFinish()
				
	def exitPM(self):
		"Exit programming mode. Vdd last exit mode"
		self.top.flushCommands()
		self.setPins(0, 0)
		self.top.hostDelay(self.delayP16)
		self.applyVPP(False)
		self.top.hostDelay(self.delayP17)
		self.applyVCC(False)
		self.applyGND(False)
		self.isInPmMode = False

	def setSDI(self, sdi):
		'''
		16 -set 24 bit sdi value
		'''
		for addr in (0x13, 0x14, 0x15):
			self.top.cmdFPGAWrite(addr, sdi & 0xFF)
			sdi = sdi >> 8


	def sendCommand(self, command):
		'''
		16 - send command
		CMD_SENDSIXINSTR	0
		CMD_SENDREGOUTINSTR	1
		CMD_ENTERPM 2
		'''
		self.top.cmdFPGAWrite(0x12, command)
	
	def readREGOUTword(self):
		def incBbAndCheckFillImage():
			self.BufferedBytes += 1
			if self.BufferedBytes == self.top.getBufferRegSize():
				self.Image += self.top.cmdReadBufferReg(self.BufferedBytes)
				self.BufferedBytes = 0
		self.busyWait()
		# self.top.hostDelay(0.000001)
		self.sendCommand(self.PROGCMD_SENDREGOUTINSTR)
		self.busyWait()
		# self.top.hostDelay(0.000001)
		self.readSDOBufferLow()
		incBbAndCheckFillImage()
		self.readSDOBufferHigh()
		incBbAndCheckFillImage()

	
	def flushBufferToImage(self):
		if self.BufferedBytes > 0:
			self.Image += self.top.cmdReadBufferReg(self.BufferedBytes)
			self.BufferedBytes = 0
	
	def sendSIX(self, instr, is9SIX=False):
		self.busyWait()		
		self.setSDI(instr)
		self.sendCommand((self.PROGCMD_SENDSIXINSTR, self.PROGCMD_SEND9SIXINSTR)[is9SIX])

	def read2words(self):
		self.executeCode((0xBA0B96, 0x000000, 0x000000))
		self.readREGOUTword()
		self.executeCode((0x0000000, 0xBA8BB6, 0x000000, 0x000000, 0xBAD3D6, 0x000000, 0x000000))
		self.readREGOUTword()
		self.executeCode((0x000000, 0xBA0BB6, 0x000000, 0x000000))
		self.readREGOUTword()
		self.executeCode((0x000000,))
		
	
	def executeCode(self, code):
		for instr in code:
			self.sendSIX(instr)

	def setPins(self, ICSPCLK=0, SDIOVALUE=0, SDIODRIVEN=1):
		'''
		16 - setPins
		'''
		data = 0
		if ICSPCLK:
			data |= 1
		if SDIODRIVEN:
			data |= 2
		if SDIOVALUE:
			data |= 4	    
		self.top.cmdFPGAWrite(0x19, data)

	def getStatusFlags(self):
		'''
		[0] - BUSY
		[1] - SDO
		'''
		self.flushBufferToImage()
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def readSDOBufferHigh(self):
		self.top.cmdFPGARead(0x10)
	
	def readSDOBufferLow(self):
		self.top.cmdFPGARead(0x13)	
	
	def rawSDIOState(self):
		return bool(self.getStatusFlags() & self.STAT_SDIO)

	def isBusy(self):
		return bool(self.getStatusFlags() & self.STAT_BUSY)
	def setTopProgrammerDelays(self):
		# print("tdel5:{:d}".format(int(math.ceil(self.delayP3 / 42e-9))))
		# print("tdly:{:d}".format(int(math.ceil(self.delayCommandDataREGOUT / 42e-9))))
		self.top.cmdFPGAWrite(0x10, int(math.ceil(self.delayP3 / 42e-9)))
		self.top.cmdFPGAWrite(0x11, int(math.ceil(self.delayCommandDataREGOUT / 42e-9)))
	def busyWait(self):
		# We do not poll the busy flag, because that would result
		# in a significant slowdown. We delay long enough for the
		# command to finish execution, instead.
		self.top.hostDelay(0.0000005)
		return
		for i in range(0, 100):
			if not self.isBusy():
				return
		       	self.top.hostDelay(0.000001)
	       	self.throwError("Timeout in busywait.")	

	def getCodeInitializeTBLPAG(self, addr, wIdx=0):
		mlw = (addr & 0xFFFF) << 4
		mmb = (addr >> 12) & 0x000FF0
		return (0x200000 | mmb, 0x880190, 0x200000 | mlw | (wIdx & 0x0F))

	def getCodeSetNVMCON(self, NVMCON):
		return (0x20000A | ((NVMCON & 0xFFFF) << 4), 0x883B0A)
	
	def isWRset(self):
		self.executeCode(self.codeExitResetVectorSimple)
		self.executeCode((0x803B02, 0x883C22, 0x000000))
		self.BufferedBytes = 0
		self.Image = ""
		self.readREGOUTword()
		self.flushBufferToImage()
		self.executeCode((0x000000,))
		return byte2int(self.Image[1]) & 0x80



"""
#    TOP2049 Open Source programming suite
#
#    Copyright (c) 2009-2017 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
__pymajor = sys.version_info[0]
__pyminor = sys.version_info[1]
if __pymajor < 2 or (__pymajor == 2 and __pyminor < 6):
	print "FATAL: TOPrammer requires Python version 2.6. Please install Python 2.6"
	sys.exit(1)


# TOPrammer version stamp
VERSION_MAJOR	= 0
VERSION_MINOR	= 17
VERSION = "%d.%d" % (VERSION_MAJOR, VERSION_MINOR)


from bitfile import *
from util import *

import time
import re

from hardware_access_usb import *
from top_devices import *
from chips import *
from user_interface import *


class FoundDev(object):
	def __init__(self, toptype, devIdentifier, busdata=None):
		self.toptype = toptype
		self.devIdentifier = devIdentifier
		self.busdata = busdata

class TOP(object):
	# Supported programmer types
	TYPE_TOP2049		= "TOP2049"

	def __init__(self, devIdentifier=None, verbose=0,
		     forceLevel=0, noqueue=False, usebroken=False,
		     forceBitfileUpload=False,
		     userInterface=ConsoleUserInterface()):

		self.verbose = verbose
		self.forceLevel = forceLevel
		self.forceBitfileUpload = forceBitfileUpload
		self.usebroken = usebroken
		self.userInterface = userInterface

		self.hw = None
		self.chip = None

		# Find the device
		devices = self.findDevices()
		if devIdentifier:
			devices = [ d for d in devices
				    if d.devIdentifier.lower() == devIdentifier.lower() ]
		if not devices:
			raise TOPException("TOP programmer device not found!")
		foundDev = devices[0] # Select first

		if noqueue:
			self.printWarning("WARNING: Command queuing disabled. " +\
				"Hardware access will be _really_ slow.")

		self.initializeProgrammer(foundDev, noqueue)

	def getProgrammerType(self):
		"Returns the TYPE_TOPxxxx"
		return self.topType

	def initializeChip(self, chipID, assignedChipOptions=()):
		"Initialize the programmer for a chip"
		# If a chip is initialized, shut it down first.
		if self.chip:
			self.shutdownChip()
		# Find the implementation of the chip.
		descriptor = ChipDescription.findOne(chipID, self.usebroken)
		self.chip = descriptor.chipImplClass.createInstance(
			top = self,
			chipDescription = descriptor,
			assignedChipOptions = assignedChipOptions)
		ok = self.configureFPGA(descriptor.bitfile, descriptor.runtimeID)
		if not ok:
			self.chip = None
			raise TOPException("Did not find BIT-file for chip implementation %s" % chipID)

	def shutdownChip(self):
		if self.chip:
			self.chip.shutdownChip()
			self.flushCommands()
			self.chip = None

	def resetChip(self):
		if self.chip:
			self.chip.shutdownChip()
			self.flushCommands()

	def getChip(self):
		"Get the chip. May return None"
		return self.chip

	def checkChip(self):
		if not self.chip:
			raise TOPException("Target chip not selected")

	def getForceLevel(self):
		return self.forceLevel

	def progressMeterInit(self, meterId, message, nrSteps):
		if self.verbose >= 1:
			self.userInterface.progressMeterInit(meterId, message, nrSteps)

	def progressMeterFinish(self, meterId):
		if self.verbose >= 1:
			self.userInterface.progressMeterFinish(meterId)

	def progressMeter(self, meterId, step):
		if self.verbose >= 1:
			self.userInterface.progressMeter(meterId, step)

	def printWarning(self, message):
		if self.verbose >= 0:
			self.flushCommands()
			self.userInterface.warningMessage(message)

	def printInfo(self, message):
		if self.verbose >= 1:
			self.flushCommands()
			self.userInterface.infoMessage(message)

	def printDebug(self, message):
		if self.verbose >= 2:
			self.flushCommands()
			self.userInterface.debugMessage(message)

	@classmethod
	def __usbdev2toptype(cls, usbdev):
		"Returns the TOP type of the USB device. None, if this is not a TOP device."
		try:
			toptype = {
				(0x2471, 0x0853):	TOP.TYPE_TOP2049,
			}[ (usbdev.idVendor, usbdev.idProduct) ]
		except (KeyError), e:
			return None
		return toptype

	@classmethod
	def findDevices(cls):
		"""Rescan all busses for TOP devices.
		Returns a list of FoundDev()"""
		usbFound = HardwareAccessUSB.scan(cls.__usbdev2toptype)
		devices = []
		for i, ud in enumerate(usbFound):
			toptype = cls.__usbdev2toptype(ud.usbdev)
			devices.append(
				FoundDev(toptype,
					 "usb:%s:%d" % (toptype, i),
					 ud)
			)
		return devices

	def initializeProgrammer(self, foundDev, noQueue):
		"Initialize the hardware"

		self.shutdownProgrammer()

		if foundDev.toptype == self.TYPE_TOP2049:
			self.hw = top2049.hardware_access.HardwareAccess(
					foundUSBDev = foundDev.busdata,
					noQueue = noQueue,
					doRawDump = (self.verbose >= 3))
			self.vcc = top2049.vcc_layouts.VCCLayout(self)
			self.vpp = top2049.vpp_layouts.VPPLayout(self)
			self.gnd = top2049.gnd_layouts.GNDLayout(self)
		else:
			assert(0)
		self.topType = foundDev.toptype

		versionRegex = (
			(r"top2049\s+ver\s*(\d+\.\d+)", self.TYPE_TOP2049),
		)

		# This is the first hardware access. Try several times since the programmer is in an unknown state.
		for _ in range(5):
			try:
				versionString = self.hw.readVersionString()
				break
			except TOPException, e:
				time.sleep(0.05)
		else:
			raise TOPException("Could not read version string from hardware")
		for (regex, t) in versionRegex:
			if t != self.topType:
				continue
			m = re.match(regex, versionString, re.IGNORECASE)
			if m:
				self.topVersion = m.group(1)
				break
		else:
			raise TOPException("Connected TOP programmer '" + versionString +\
				"' is not supported by Toprammer, yet")
		self.printInfo("Initializing the " + self.topType + " version " + self.topVersion)

		self.hw.hardwareInit()

	def shutdownProgrammer(self):
		if self.hw:
			self.hw.shutdown()
		self.hw = None
		self.topType = None
		self.topVersion = None

	def __readBitfileID(self):
		self.cmdFPGARead(0xFD)
		self.cmdFPGARead(0xFE)
		self.cmdFPGARead(0xFF)
		data = self.cmdReadBufferReg(3)
		gotID = byte2int(data[0]) | (byte2int(data[1]) << 8)
		if gotID == 0xFEFD or gotID == 0xFFFF:
			gotID = 0
		gotRev = byte2int(data[2])
		if gotRev == 0xFF:
			gotRev = 0
		return (gotID, gotRev)

	def __bitfileUpload(self, requiredRuntimeID):
		(requiredID, requiredRevision) = requiredRuntimeID
		if requiredID and requiredRevision and not self.forceBitfileUpload:
			# Check if the bitfile is already uploaded.
			(gotID, gotRev) = self.__readBitfileID()
			if gotID == requiredID and gotRev == requiredRevision:
				self.printDebug("Bitfile %s ID 0x%04X Rev 0x%02X is already uploaded." %\
						(self.bitfile.getFilename(), gotID, gotRev))
				return
			self.printDebug("Current runtime ID 0x%04X Rev 0x%02X. Uploading new bitfile..." %\
					(gotID, gotRev))

		self.printDebug("Uploading bitfile %s..." % self.bitfile.getFilename())

		self.hw.FPGAInitiateConfig()
		data = self.bitfile.getPayload()
		chunksz = self.hw.getFPGAMaxConfigChunkSize()
		chunksz = chunksz if chunksz > 0 else len(data)
		for i in range(0, len(data), chunksz):
			self.hw.FPGAUploadConfig(i, data[i : i + chunksz])
		self.flushCommands()

		if requiredID and requiredRevision:
			# Check the uploaded ID
			(gotID, gotRev) = self.__readBitfileID()
			if gotID != requiredID or gotRev != requiredRevision:
				raise TOPException("bit-upload: The bitfile upload succeed, "
					"but the read ID or revision is invalid. "
					"(Got 0x%04X/0x%02X, but expected 0x%04X/0x%02X)" %\
					(gotID, gotRev, requiredID, requiredRevision))

	def configureFPGA(self, bitfileName, runtimeIDs):
		"""Upload and configure the FPGA.
		bitfileName -> The name of the bitfile to upload
		runtimeIDs -> bottom-half ID tuple: (majorID, minorID)"""
		# Get the bitfile path.
		bitfilePath = bitfileFind(bitfileName)
		if not bitfilePath:
			return False
		# Open and parse the bitfile.
		self.bitfile = Bitfile()
		self.bitfile.parseFile(bitfilePath)
		# Initialize the hardware.
		self.__bitfileUpload(runtimeIDs)
		return True

	def readSignature(self):
		"""Reads the chip signature and returns it."""
		self.printDebug("Reading signature from chip...")
		self.checkChip()
		sig = self.chip.readSignature()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(sig))
		return sig

	def eraseChip(self):
		"""Erase the chip."""
		self.printDebug("Erasing chip...")
		self.checkChip()
		self.chip.erase()
		self.flushCommands()

	def testChip(self):
		"""Run a unit-test on the chip."""
		self.printDebug("Running chip unit-test...")
		self.checkChip()
		self.chip.test()
		self.flushCommands()
		self.printInfo("Chip unit-test terminated successfully.")

	def readProgmem(self):
		"""Reads the program memory image and returns it."""
		self.printDebug("Reading program memory from chip...")
		self.checkChip()
		image = self.chip.readProgmem()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeProgmem(self, image):
		"""Writes a program memory image to the chip."""
		self.printDebug("Writing %d bytes of program memory to chip..." % len(image))
		self.checkChip()
		self.chip.writeProgmem(image)
		self.flushCommands()
		self.printDebug("Done writing image.")

	def readEEPROM(self):
		"""Reads the EEPROM image and returns it."""
		self.printDebug("Reading EEPROM from chip...")
		self.checkChip()
		image = self.chip.readEEPROM()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeEEPROM(self, image):
		"""Writes an EEPROM image to the chip."""
		self.printDebug("Writing %d bytes of EEPROM to chip..." % len(image))
		self.checkChip()
		self.chip.writeEEPROM(image)
		self.flushCommands()
		self.printDebug("Done writing image.")

	def readFuse(self):
		"""Reads the fuses image and returns it."""
		self.printDebug("Reading fuses from chip...")
		self.checkChip()
		image = self.chip.readFuse()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeFuse(self, image):
		"""Writes a fuses image to the chip."""
		self.printDebug("Writing %d bytes of fuses to chip..." % len(image))
		self.checkChip()
		self.chip.writeFuse(image)
		self.flushCommands()
		self.printDebug("Done writing image.")

	def readLockbits(self):
		"""Reads the Lock bits image and returns it."""
		self.printDebug("Reading lock-bits from chip...")
		self.checkChip()
		image = self.chip.readLockbits()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeLockbits(self, image):
		"""Writes a Lock bits image to the chip."""
		self.printDebug("Writing %d bytes of lock-bits to chip..." % len(image))
		self.checkChip()
		self.chip.writeLockbits(image)
		self.flushCommands()
		self.printDebug("Done writing image.")

	def readRAM(self):
		"""Reads the RAM and returns it."""
		self.printDebug("Reading RAM from chip...")
		self.checkChip()
		image = self.chip.readRAM()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeRAM(self, image):
		"""Writes the RAM image to the chip."""
		self.printDebug("Writing %d bytes of RAM to the chip..." % len(image))
		self.checkChip()
		self.chip.writeRAM(image)
		self.flushCommands()
		self.printDebug("Done writing the image.")
		
	def readUserIdLocation(self):
		"""Reads the User ID Location and returns it."""
		self.printDebug("Reading UIL from chip...")
		self.checkChip()
		image = self.chip.readUserIdLocation()
		self.flushCommands()
		self.printDebug("Done reading %d bytes." % len(image))
		return image

	def writeUserIdLocation(self, image):
		"""Writes the User ID Location image to the chip."""
		self.printDebug("Writing %d bytes to UIL of the chip..." % len(image))
		self.checkChip()
		self.chip.writeUserIdLocation(image)
		self.flushCommands()
		self.printDebug("Done writing the image.")		

	def cmdDelay(self, seconds):
		"""Send a delay request to the device. Note that this causes the
		programmer to execute the delay. For a host-delay, use hostDelay()"""
		self.hw.delay(seconds)

	def hostDelay(self, seconds):
		"""Flush all commands and delay the host computer for 'seconds'"""
		self.flushCommands(seconds)

	def getOscillatorHz(self):
		"""Returns the FPGA oscillator frequency, in Hz.
		The oscillator is connected to the FPGA clk pin."""
		return self.hw.getOscillatorHz()

	def getBufferRegSize(self):
		"""Returns the size (in bytes) of the buffer register."""
		return self.hw.getBufferRegSize()

	def cmdReadBufferReg(self, nrBytes=-1):
		"""Read the buffer register. Returns nrBytes (default all bytes)."""
		regSize = self.getBufferRegSize()
		if nrBytes < 0:
			nrBytes = regSize
		assert(nrBytes <= regSize)
		if not nrBytes:
			return b""
		return self.hw.readBufferReg(nrBytes)

	def cmdReadBufferReg8(self):
		"""Read a 8bit value from the buffer register."""
		stat = self.cmdReadBufferReg(1)
		stat = byte2int(stat[0])
		return stat

	def cmdReadBufferReg16(self):
		"""Read a 16bit value from the buffer register."""
		stat = self.cmdReadBufferReg(2)
		stat = byte2int(stat[0]) | (byte2int(stat[1]) << 8)
		return stat

	def cmdReadBufferReg24(self):
		"""Read a 24bit value from the buffer register."""
		stat = self.cmdReadBufferReg(3)
		stat = byte2int(stat[0]) | (byte2int(stat[1]) << 8) | (byte2int(stat[2]) << 16)
		return stat

	def cmdReadBufferReg32(self):
		"""Read a 32bit value from the buffer register."""
		stat = self.cmdReadBufferReg(4)
		stat = byte2int(stat[0]) | (byte2int(stat[1]) << 8) | \
		       (byte2int(stat[2]) << 16) | (byte2int(stat[3]) << 24)
		return stat

	def cmdReadBufferReg48(self):
		"""Read a 48bit value from the buffer register."""
		stat = self.cmdReadBufferReg(6)
		stat = byte2int(stat[0]) | (byte2int(stat[1]) << 8) | \
		       (byte2int(stat[2]) << 16) | (byte2int(stat[3]) << 24) | \
		       (byte2int(stat[4]) << 32) | (byte2int(stat[5]) << 40)
		return stat

	def cmdFPGARead(self, address):
		"""Read a byte from the FPGA at address into the buffer register."""
		self.hw.FPGARead(address)

	def cmdFPGAWrite(self, address, byte):
		"""Write a byte to an FPGA address."""
		return self.hw.FPGAWrite(address, byte)

	def cmdLoadGNDLayout(self, layout):
		"""Load the GND configuration into the programmer."""
		self.hw.loadGNDLayout(layout)

	def cmdSetVPPVoltage(self, voltage):
		"""Set the VPP voltage. voltage is a floating point voltage number."""
		self.hw.setVPPVoltage(voltage)

	def cmdLoadVPPLayout(self, layout):
		"""Load the VPP configuration into the programmer."""
		self.hw.loadVPPLayout(layout)

	def cmdSetVCCVoltage(self, voltage):
		"""Set the VCC voltage. voltage is a floating point voltage number."""
		self.hw.setVCCVoltage(voltage)

	def cmdLoadVCCLayout(self, layout):
		"""Load the VCC configuration into the shift registers."""
		self.hw.loadVCCLayout(layout)

	def cmdEnableZifPullups(self, enable):
		"""Enable the ZIF socket signal pullups."""
		self.hw.enableZifPullups(enable)

	def flushCommands(self, sleepSeconds=0):
		"""Flush command queue and optionally sleep for 'sleepSeconds'."""
		self.hw.flushCommands(sleepSeconds)



"""
#    TOP2049 Open Source programming suite
#
#    Chip support
#
#    Copyright (c) 2009-2017 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *
from layout_generator import *
from user_interface import *
from generic_algorithms import *
from ihex import *


class Chip(object):
	SUPPORT_ERASE		= (1 << 0)
	SUPPORT_SIGREAD		= (1 << 1)
	SUPPORT_PROGMEMREAD	= (1 << 2)
	SUPPORT_PROGMEMWRITE	= (1 << 3)
	SUPPORT_EEPROMREAD	= (1 << 4)
	SUPPORT_EEPROMWRITE	= (1 << 5)
	SUPPORT_FUSEREAD	= (1 << 6)
	SUPPORT_FUSEWRITE	= (1 << 7)
	SUPPORT_LOCKREAD	= (1 << 8)
	SUPPORT_LOCKWRITE	= (1 << 9)
	SUPPORT_RAMREAD		= (1 << 10)
	SUPPORT_RAMWRITE	= (1 << 11)
	SUPPORT_TEST		= (1 << 12)
	SUPPORT_UILREAD		= (1 << 13)
	SUPPORT_UILWRITE	= (1 << 14)

	@classmethod
	def chipSupportsAttr(cls, methodName):
		"""Check if a chip implementation supports a feature.
		'methodName' is the class member function to check for"""
		# This works by checking whether the subclass overloaded
		# the member function attribute.
		return getattr(cls, methodName) != getattr(Chip, methodName)

	@classmethod
	def getSupportFlags(cls):
		"Get the SUPPORT_... flags for this chip"
		flags = 0
		for (methodName, flag) in (
				("erase",		cls.SUPPORT_ERASE),
				("readSignature",	cls.SUPPORT_SIGREAD),
				("readProgmem",		cls.SUPPORT_PROGMEMREAD),
				("writeProgmem",	cls.SUPPORT_PROGMEMWRITE),
				("readEEPROM",		cls.SUPPORT_EEPROMREAD),
				("writeEEPROM",		cls.SUPPORT_EEPROMWRITE),
				("readFuse",		cls.SUPPORT_FUSEREAD),
				("writeFuse",		cls.SUPPORT_FUSEWRITE),
				("readLockbits",	cls.SUPPORT_LOCKREAD),
				("writeLockbits",	cls.SUPPORT_LOCKWRITE),
				("readRAM",		cls.SUPPORT_RAMREAD),
				("writeRAM",		cls.SUPPORT_RAMWRITE),
				("test",		cls.SUPPORT_TEST),
				("readUserIdLocation",	cls.SUPPORT_UILREAD),
				("writeUserIdLocation",		cls.SUPPORT_UILWRITE)):
			
			if cls.chipSupportsAttr(methodName):
				flags |= flag
		return flags

	@classmethod
	def createInstance(cls, top, chipDescription, assignedChipOptions=()):
		instance = cls()
		instance.top = top
		instance.chipDescription = chipDescription
		for acopt in assignedChipOptions:
			copt = chipDescription.getChipOption(acopt.name)
			if not copt:
				raise TOPException("'%s' is not a valid chip option "
					"for chip ID '%s'" %\
					(acopt.name, chipDescription.chipID))
			acopt.detectAndVerifyType(copt)
		instance.assignedChipOptions = assignedChipOptions
		instance.generateVoltageLayouts()
		return instance

	def __init__(self, chipPackage=None, chipPinVCC=None, chipPinsVPP=None, chipPinGND=None):
		"""chipPackage is the ID string for the package.
		May be None, if no initial auto-layout is required.
		chipPinVCC is the required VCC pin on the package.
		chipPinsVPP is the required VPP pin on the package.
		chipPinGND is the required GND pin on the package."""

		self.__chipPackage = chipPackage
		self.__chipPinVCC = chipPinVCC
		self.__chipPinsVPP = chipPinsVPP
		self.__chipPinGND = chipPinGND

	def printWarning(self, message):
		self.top.printWarning(self.chipDescription.chipID +\
				      ": Warning - " + message)

	def printInfo(self, message):
		self.top.printInfo(self.chipDescription.chipID +\
				   ": " + message)

	def printDebug(self, message):
		self.top.printDebug(self.chipDescription.chipID +\
				    ": Debug - " + message)

	def throwError(self, message, always=False):
		message = self.chipDescription.chipID + ": " + message
		if self.top.getForceLevel() >= 1 and not always:
			self.printWarning(message)
		else:
			raise TOPException(message)

	def generateVoltageLayouts(self):
		if self.__chipPackage:
			self.generator = createLayoutGenerator(self.__chipPackage)
			self.generator.setProgrammerType(self.top.getProgrammerType())
			self.generator.setPins(vccPin=self.__chipPinVCC,
					       vppPins=self.__chipPinsVPP,
					       gndPin=self.__chipPinGND)
			self.generator.recalculate()

	def getLayoutGenerator(self):
		if self.__chipPackage:
			return self.generator
		return None

	def applyVCC(self, turnOn):
		"Turn VCC on, using the auto-layout."
		if turnOn:
			try:
				generator = self.generator
			except (AttributeError), e:
				self.throwError("BUG: Using auto-layout, but did not initialize it.",
						always=True)
			generator.applyVCCLayout(self.top)
		else:
			self.top.vcc.setLayoutMask(0)

	def applyVPP(self, turnOn, packagePinsToTurnOn=[]):
		"""Turn VPP on, using the auto-layout.
		packagePinsToTurnOn is a list of pins on the package to drive to VPP.
		If it is not passed (or an empty list), all VPP pins of the layout are turned on.
		The parameter is unused, if turnOn=False."""
		if turnOn:
			try:
				generator = self.generator
			except (AttributeError), e:
				self.throwError("BUG: Using auto-layout, but did not initialize it.",
						always=True)
			generator.applyVPPLayout(self.top, packagePinsToTurnOn)
		else:
			self.top.vpp.setLayoutMask(0)

	def applyGND(self, turnOn):
		"Turn GND on, using the auto-layout."
		if turnOn:
			try:
				generator = self.generator
			except (AttributeError), e:
				self.throwError("BUG: Using auto-layout, but did not initialize it.",
						always=True)
			generator.applyGNDLayout(self.top)
		else:
			self.top.gnd.setLayoutMask(0)

	def progressMeterInit(self, message, nrSteps):
		self.top.progressMeterInit(AbstractUserInterface.PROGRESSMETER_CHIPACCESS,
					   message, nrSteps)

	def progressMeterFinish(self):
		self.top.progressMeterFinish(AbstractUserInterface.PROGRESSMETER_CHIPACCESS)

	def progressMeter(self, step):
		self.top.progressMeter(AbstractUserInterface.PROGRESSMETER_CHIPACCESS,
				       step)

	def getChipOptionValue(self, name, default=None):
		"""Get an AssignedChipOption value that was set by the user.
		If no such option is found, it returns 'default'."""
		name = name.lower()
		for acopt in self.assignedChipOptions:
			if acopt.name.lower() == name:
				return acopt.value
		return default

	def shutdownChip(self):
		# Override me in the subclass, if required.
		self.printDebug("Default chip shutdown")
		GenericAlgorithms(self).simpleVoltageShutdown()

	def getIHexInterpreter(self):
		# Returns the IHEX file interpreter.
		# This defaults to the non-section interpreter.
		# Override me in the subclass, if required.
		return IHexInterpreter()

	def readSignature(self):
		# Override me in the subclass, if required.
		self.throwError("Signature reading not supported",
				always=True)

	def erase(self):
		# Override me in the subclass, if required.
		self.throwError("Chip erasing not supported",
				always=True)

	def test(self):
		# Override me in the subclass, if required.
		self.throwError("Chip testing not supported",
				always=True)

	def readProgmem(self):
		# Override me in the subclass, if required.
		self.throwError("Program memory reading not supported",
				always=True)

	def writeProgmem(self, image):
		# Override me in the subclass, if required.
		self.throwError("Program memory writing not supported",
				always=True)

	def readEEPROM(self):
		# Override me in the subclass, if required.
		self.throwError("EEPROM reading not supported",
				always=True)

	def writeEEPROM(self, image):
		# Override me in the subclass, if required.
		self.throwError("EEPROM writing not supported",
				always=True)

	def readFuse(self):
		# Override me in the subclass, if required.
		self.throwError("Fuse reading not supported",
				always=True)

	def writeFuse(self, image):
		# Override me in the subclass, if required.
		self.throwError("Fuse writing not supported",
				always=True)

	def readLockbits(self):
		# Override me in the subclass, if required.
		self.throwError("Lockbit reading not supported",
				always=True)

	def writeLockbits(self, image):
		# Override me in the subclass, if required.
		self.throwError("Lockbit writing not supported",
				always=True)

	def readRAM(self):
		# Override me in the subclass, if required.
		self.throwError("RAM reading not supported",
				always=True)

	def writeRAM(self, image):
		# Override me in the subclass, if required.
		self.throwError("RAM writing not supported",
				always=True)
		
	def readUserIdLocation(self):
		# Override me in the subclass, if required.
		self.throwError("User ID Location reading not supported",
				always=True)

	def writeUserIdLocation(self, image):
		# Override me in the subclass, if required.
		self.throwError("User ID Location writing not supported",
				always=True)

__registeredChips = []

def getRegisteredChips():
	"Get a list of registered ChipDescriptions"
	return __registeredChips

def getRegisteredVendors():
	"Returns a dict of 'vendor : [descriptor, ...]' "
	vendors = { }
	for descriptor in getRegisteredChips():
		for vendor in descriptor.chipVendors:
			vendors.setdefault(vendor, []).append(descriptor)
	return vendors

def _registerChip(chipDesc):
	regList = getRegisteredChips()
	if chipDesc.chipID in [ cd.chipID for cd in regList ]:
		raise TOPException("Chip description registration: "
			"The chipID '%s' is not unique." %\
			chipDesc.chipID)
	regList.append(chipDesc)

class BitDescription:
	def __init__(self, bitNr, description):
		self.bitNr = bitNr
		self.description = description

class ChipOption(object):
	TYPE_UNKNOWN	= "unknown"
	TYPE_BOOL	= "bool"
	TYPE_INT	= "int"
	TYPE_FLOAT	= "float"

	def __init__(self, optType, name, description=""):
		self.optType = optType
		self.name = name
		self.description = description

	def __repr__(self):
		desc = " / " + self.description if self.description else ""
		return "%s (%s%s)" % (self.name, self.optType, desc)

	def castValue(self, string):
		return None

class ChipOptionBool(ChipOption):
	def __init__(self, name, description=""):
		ChipOption.__init__(self, ChipOption.TYPE_BOOL,
				    name, description)

	def castValue(self, string):
		return str2bool(string)

class ChipOptionInt(ChipOption):
	def __init__(self, name, description="", minVal=None, maxVal=None):
		ChipOption.__init__(self, ChipOption.TYPE_INT,
				    name, description)
		self.minVal = int(minVal) if minVal is not None else None
		self.maxVal = int(maxVal) if minVal is not None else None

	def __limitError(self, value):
		raise TOPException("%s: Value exceeds limits: %d <= %d <= %d" %\
			(self.name, self.minVal, value, self.maxVal))

	def castValue(self, string):
		try:
			value = int(string)
		except (ValueError), e:
			return None
		if (self.minVal is not None and value < self.minVal) or\
		   (self.maxVal is not None and value > self.maxVal):
			self.__limitError(value)
		return value

class ChipOptionFloat(ChipOption):
	def __init__(self, name, description="", minVal=None, maxVal=None):
		ChipOption.__init__(self, ChipOption.TYPE_FLOAT,
				    name, description)
		self.minVal = float(minVal) if minVal is not None else None
		self.maxVal = float(maxVal) if minVal is not None else None

	def __limitError(self, value):
		raise TOPException("%s: Value exceeds limits: %f <= %f <= %f" %\
			(self.name, self.minVal, value, self.maxVal))

	def castValue(self, string):
		try:
			value = float(string)
		except (ValueError), e:
			return None
		if (self.minVal is not None and value < self.minVal) or\
		   (self.maxVal is not None and value > self.maxVal):
			self.__limitError(value)
		return value

class AssignedChipOption(ChipOption):
	def __init__(self, name, value):
		ChipOption.__init__(self, ChipOption.TYPE_UNKNOWN, name)
		self.value = value

	def detectAndVerifyType(self, baseOption):
		assert(baseOption.optType != self.TYPE_UNKNOWN)
		if self.optType != self.TYPE_UNKNOWN:
			return
		value = baseOption.castValue(self.value)
		if value is None:
			raise TOPException("Chip option '%s' type mismatch. "
				"Must be '%s'." %\
				(self.name, baseOption.optType))
		self.optType = baseOption.optType
		self.value = value

class ChipDescription:
	# Possible chip types
	TYPE_MCU	= 0	# Microcontroller
	TYPE_EPROM	= 1	# EPROM
	TYPE_EEPROM	= 2	# EEPROM
	TYPE_GAL	= 3	# PAL/GAL
	TYPE_SRAM	= 4	# Static RAM
	TYPE_LOGIC	= 5	# Logics chip
	TYPE_INTERNAL	= 999	# For internal use only

	def __init__(self, chipImplClass, bitfile, chipID="",
		     runtimeID=(0,0),
		     chipType=TYPE_MCU,
		     chipVendors="Other",
		     description="", fuseDesc=(), lockbitDesc=(),
		     packages=None, comment="",
		     chipOptions=(),
		     maintainer="Michael Buesch <m@bues.ch>",
		     broken=False):
		"""Chip implementation class description.
		chipImplClass	=> The implementation class of the chip.
		bitfile		=> The name of the default bitfile loaded on initialization.
		chipID		=> The chip-ID string. Will default to the bitfile ID string.
		runtimeID	=> The runtime-ID is a tuple of two numbers that uniquely
				   identifies a loaded FPGA configuration. The first number in the
				   tuple is an ID number and the second number is a revision number.
		chipType	=> Chip type. Defaults to MCU.
		chipVendors	=> The chip vendor name(s).
		description	=> Human readable chip description string.
		fuseDesc	=> Tuple of fuse bits descriptions (BitDescription(), ...)
		lockbitDesc	=> Tuple of lock bits descriptions (BitDescription(), ...)
		packages	=> List of supported packages.
				   Each entry is a tuple of two strings: ("PACKAGE", "description")
		comment		=> Additional comment string.
		chipOptions	=> Tuple of ChipOption instances, if any.
		maintainer	=> Maintainer name.
		broken		=> Boolean flag to mark the implementation as broken.
		"""

		if not chipID:
			chipID = bitfile
		if isinstance(chipVendors, str):
			chipVendors = (chipVendors, )
		self.chipImplClass = chipImplClass
		self.bitfile = bitfile
		self.chipID = chipID
		self.runtimeID = runtimeID
		self.chipType = chipType
		self.chipVendors = chipVendors
		self.description = description
		self.fuseDesc = fuseDesc
		self.lockbitDesc = lockbitDesc
		self.packages = packages
		self.comment = comment
		self.chipOptions = chipOptions
		self.maintainer = maintainer
		self.broken = broken

		_registerChip(self)

	@classmethod
	def findAll(cls, chipID, allowBroken=False):
		"Find all ChipDescriptions by fuzzy chipID match."
		found = [ chipDesc for chipDesc in getRegisteredChips() if (
			(not chipDesc.broken or allowBroken) and\
			(chipDesc.chipID.lower().find(chipID.lower()) >= 0)
		) ]
		return found

	@classmethod
	def findOne(cls, chipID, allowBroken=False):
		"""Find a chip implementation and return the ChipDescriptor.
		Raise an exception, if chipID is not unique."""
		found = cls.findAll(chipID, allowBroken)
		if not found:
			raise TOPException("Did not find chipID \"%s\"" % chipID)
		if len(found) != 1:
			choices = [ desc.chipID for desc in found ]
			raise TOPException(
				"The chipID \"%s\" is not unique. Choices are: %s" %\
				(chipID, ", ".join(choices)))
		return found[0]

	@classmethod
	def dumpAll(cls, fd, verbose=1, showBroken=True):
		"Dump all supported chips to file fd."
		count = 0
		for chip in getRegisteredChips():
			if chip.broken and not showBroken:
				continue
			if chip.chipType == cls.TYPE_INTERNAL:
				continue
			count = count + 1
			if count >= 2:
				fd.write("\n")
			chip.dump(fd, verbose)

	def dump(self, fd, verbose=1):
		"Dump information about a registered chip to file fd."

		if verbose <= 0:
			fd.write(self.chipID)
			return

		def wrline(prefix, content):
			# Write a formatted feature line
			fd.write("%15s:  %s\n" % (prefix, content))

		banner = ""
		if self.chipVendors:
			banner += ", ".join(self.chipVendors) + "  "
		if self.description:
			banner += self.description
		else:
			banner += self.bitfile
		extraFlags = []
		if not self.maintainer:
			extraFlags.append("Orphaned")
		if self.broken:
			extraFlags.append("Broken implementation")
		if extraFlags:
			banner += "  (%s)" % "; ".join(extraFlags)
		sep = '+' + '-' * (len(banner) + 4) + '+\n'
		fd.write(sep + '|  ' + banner + '  |\n' + sep)

		if verbose >= 1:
			wrline("Chip ID", self.chipID)
		if verbose >= 2:
			bitfile = self.bitfile
			if not bitfile.endswith('.bit'):
				bitfile += '.bit'
			wrline("BIT file", bitfile)
		if verbose >= 1:
			for opt in self.chipOptions:
				wrline("Chip option", str(opt))
		if verbose >= 3 and self.packages:
			for (package, description) in self.packages:
				if description:
					description = "  (" + description + ")"
				wrline("Chip package", package + description)
		if verbose >= 4:
			supportedFeatures = (
				(Chip.SUPPORT_ERASE,		"Full chip erase"),
				(Chip.SUPPORT_SIGREAD,		"Chip signature reading"),
				(Chip.SUPPORT_PROGMEMREAD,	"Program memory reading (flash)"),
				(Chip.SUPPORT_PROGMEMWRITE,	"Program memory writing (flash)"),
				(Chip.SUPPORT_EEPROMREAD,	"(E)EPROM memory reading"),
				(Chip.SUPPORT_EEPROMWRITE,	"(E)EPROM memory writing"),
				(Chip.SUPPORT_FUSEREAD,		"Fuse bits reading"),
				(Chip.SUPPORT_FUSEWRITE,	"Fuse bits writing"),
				(Chip.SUPPORT_LOCKREAD,		"Lock bits reading"),
				(Chip.SUPPORT_LOCKWRITE,	"Lock bits writing"),
				(Chip.SUPPORT_RAMREAD,		"RAM reading"),
				(Chip.SUPPORT_RAMWRITE,		"RAM writing"),
				(Chip.SUPPORT_TEST,		"Unit-testing"),
				(Chip.SUPPORT_UILMREAD,		"User ID Location reading"),
				(Chip.SUPPORT_UILWRITE,		"User ID Location writing"),
			)
			supportFlags = self.chipImplClass.getSupportFlags()
			for (flag, description) in supportedFeatures:
				if flag & supportFlags:
					wrline("Support for", description)
		if verbose >= 2 and self.comment:
			wrline("Comment", self.comment)
		if verbose >= 3:
			maintainer = self.maintainer
			if not maintainer:
				maintainer = "NONE"
			wrline("Maintainer", maintainer)

	def getChipOption(self, name):
		"Get a ChipOption by case insensitive 'name'."
		name = name.lower()
		for opt in self.chipOptions:
			if opt.name.lower() == name:
				return opt
		return None



"""
#    TOP2049 Open Source programming suite
#
#    Atmel AT89C2051 DIP20 Support
#
#    Copyright (c) 2010 Guido
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_AT89C2051dip20(Chip):
	STAT_BUSY	= 0x01 # Programmer is running a command
	STAT_ERR	= 0x02 # Error during write

	def __init__(self):
		Chip.__init__(self,
			      chipPackage = "DIP20",
			      chipPinVCC = 20,
			      chipPinsVPP = 1,
			      chipPinGND = 10)

	def __initChip(self):
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(True)
		self.top.cmdSetVCCVoltage(5)
		self.top.cmdSetVPPVoltage(5)

	def readSignature(self):
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.__loadCommand(1) # set P3.2
		self.__setP3x(P33=0, P34=0, P35=0, IA=0)
		data = ""
		self.top.cmdFPGARead(0x10)
		self.__setP3x(P33=0, P34=0, P35=0, IA=1)
		self.__setP3x(P33=0, P34=0, P35=0, IA=0)
		self.top.cmdFPGARead(0x10)
		self.__setP3x(P33=0, P34=0, P35=0, IA=1)
		self.__setP3x(P33=0, P34=0, P35=0, IA=0)
		self.top.cmdFPGARead(0x10)
		data += self.top.cmdReadBufferReg()
		self.__setP3x(P33=0, P34=1, P35=0, IA=0)
		self.__loadCommand(6) # VPP off
		signature = ""
		signature += data[0]
		signature += data[1]
		self.top.printInfo("Signature: %X, %X"  % (byte2int(signature[0]), byte2int(signature[1])))
		return signature

	def erase(self):
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1) # set P3.2
		self.top.cmdSetVPPVoltage(5)
		self.applyVPP(True)
		self.__loadCommand(5) # VPP on
		self.__setP3x(P33=1, P34=0, P35=0, IA=0)
		self.top.cmdSetVPPVoltage(12)
		self.__runCommandSync(4)
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(5)
		self.__setP3x(P33=0, P34=1, P35=0, IA=0)
		self.__loadCommand(5) # VPP off
		self.top.flushCommands()
		self.top.printInfo("at89c2051dip20: Erasing flash, verifying ...")
		ok = self.__verifyErase()
		if ok == 0:
			self.top.printInfo("at89c2051dip20: Erase done.")
		else:
			self.top.printInfo("at89c2051dip20: Erase failed!")

	def readProgmem(self):
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1) # set P3.2
		self.top.cmdSetVPPVoltage(5)
		self.applyVPP(True)
		self.__loadCommand(5) # VPP on
		self.__setP3x(P33=0, P34=0, P35=1, IA=0)
		image = ""
		byteCount = 0
		self.progressMeterInit("Reading Flash", 0x800)
		for addr in range(0, 0x800):
			self.progressMeter(addr)
			self.top.cmdFPGARead(0x10)
			self.__setP3x(P33=0, P34=0, P35=1, IA=1)
			self.__setP3x(P33=0, P34=0, P35=1, IA=0)
			byteCount += 1
			if byteCount == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(byteCount)
				byteCount = 0
		image += self.top.cmdReadBufferReg(byteCount)
		self.applyVPP(False)
		self.__setP3x(P33=0, P34=1, P35=0, IA=0)
		self.__loadCommand(5) # VPP off
		self.top.flushCommands()
		self.progressMeterFinish()

		return image

	def writeProgmem(self, image):
		if len(image) > 0x800:
			self.throwError("Invalid EPROM image size %d (expected <=%d)" %\
				(len(image), 0x800))
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1) # set P3.2
		self.top.cmdSetVPPVoltage(5)
		self.applyVPP(True)
		self.__loadCommand(5) # VPP on
		self.__setP3x(P33=0, P34=1, P35=1, IA=0)
		self.top.cmdSetVPPVoltage(12)
		self.progressMeterInit("Writing Flash", len(image))
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			data = byte2int(image[addr])
			if data != 0xFF:
				self.__loadData(data)
				self.__loadCommand(3)
				ok = self.__progWait()
				if (ok & self.STAT_ERR) != 0:
					self.throwError("Write byte failed.")
			self.__setP3x(P33=0, P34=1, P35=1, IA=1)
			self.__setP3x(P33=0, P34=1, P35=1, IA=0)
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(5)
		self.__setP3x(P33=0, P34=1, P35=0, IA=0)
		self.__loadCommand(5) # VPP off
		self.top.flushCommands()
		self.progressMeterFinish()
		ok = self.__verifyProgmem(image)
		if ok == 0:
			self.top.printInfo("at89c2051dip20: Write flash done.")
		else:
			self.top.printInfo("at89c2051dip20: Write flash failed!")

	def __verifyErase(self):
		ok = 0
		image = self.readProgmem()
		for addr in range(0, 0x800):
			if byte2int(image[addr]) != 0xFF:
				ok = 1
		return ok

	def __verifyProgmem(self,image):
		data = self.readProgmem()
		ok = 0
		for addr in range(0, 0x800):
			if byte2int(image[addr]) != byte2int(data[addr]):
				ok = 1
		return ok

	def __loadData(self, data):
		self.top.cmdFPGAWrite(0x10, data)

	def __loadCommand(self, command):
		self.top.cmdFPGAWrite(0x12, command & 0xFF)

	def __runCommandSync(self, command):
		self.__loadCommand(command)
		self.__busyWait()

	def __setP3x(self, P33, P34, P35, IA):
		data = 0
		if P33:
			data |= 1
		if P34:
			data |= 2
		if P35:
			data |= 4
		if IA:
			data |= 8
		self.top.cmdFPGAWrite(0x16, data)

	def __getStatusFlags(self):
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def __busy(self):
		return bool(self.__getStatusFlags() & self.STAT_BUSY)

	def __busyWait(self):
		for i in range(0, 26):
			if not self.__busy():
				return
			self.top.hostDelay(0.001)
		self.throwError("Timeout in busywait.")

	def __progWait(self):
		for i in range(0,4):
			self.top.cmdFPGARead(0x12)
			stat = self.top.cmdReadBufferReg()
			if (byte2int(stat[0]) & self.STAT_BUSY) == 0:
				return byte2int(stat[0])
			self.top.hostDelay(0.001)
		self.throwError("Timeout in busywait.")

ChipDescription(
	Chip_AT89C2051dip20,
	bitfile = "at89c2051dip20",
	runtimeID = (0x0005, 0x01),
	chipVendors = "Atmel",
	description = "AT89C2051",
	maintainer = None,
	packages = ( ("DIP20", ""), )
)



"""
#    TOP2049 Open Source programming suite
#
#    TOP2049 GND layout definitions
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
if __name__ == "__main__":
	sys.path.insert(0, sys.path[0] + "/../..")
from libtoprammer.generic_layout import *


class GNDLayout(GenericLayout):
	# A list of valid ZIF GND pins (0=none)
	validPins = (0, 5, 14, 15, 16, 17, 18, 19, 20, 24, 26, 27,
		28, 29, 33, 34, 35)

	def __init__(self, top=None):
		GenericLayout.__init__(self, nrZifPins=48)
		self.top = top
		self.layouts = []
		for pin in self.validPins:
			id = pin
			if id != 0:
				id -= 4
			mask = 0
			if pin != 0:
				mask |= (1 << (pin - 1))
			self.layouts.append( (id, mask) )

	def supportedLayouts(self):
		return self.layouts

	def setLayoutID(self, id):
		self.top.cmdLoadGNDLayout(id)

if __name__ == "__main__":
	print "ZIF socket GND layouts"
	print GNDLayout()



"""
#    TOP2049 Open Source programming suite
#
#    Atmel AT89C2051 DIP20 Support
#
#    Copyright (c) 2010 Guido
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_AT89S52dip40(Chip):
	STAT_BUSY	= 0x01 # Programmer is running a command
	STAT_ERR	= 0x02 # Error during write

	def __init__(self):
		Chip.__init__(self,
			      chipPackage = "DIP40",
			      chipPinVCC = 40,
			      chipPinsVPP = 31,
			      chipPinGND = 20)
		self.flashPageSize = 0x4000
		self.flashPages = 1

	def __initChip(self):
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(True)
		self.top.cmdSetVCCVoltage(5)
		self.top.cmdSetVPPVoltage(5)

	def readSignature(self):
		self.__initChip()
		self.top.cmdEnableZifPullups(True)
		self.applyGND(True)
		self.applyVCC(True)
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.__loadCommand(1) # set nPROG
		self.__loadAddress(0x0100)
		self.__setPx()
		data = ""
		self.top.cmdFPGARead(0x10)
		self.__loadAddress(0x0200)
		self.top.cmdFPGARead(0x10)		
		data += self.top.cmdReadBufferReg()
		self.applyVPP(False)
		self.__loadCommand(6) # VPP off
		signature = ""
		signature += data[0]
		signature += data[1]
		self.top.printInfo("Signature: %X, %X"  % (byte2int(signature[0]), byte2int(signature[1])))
		return signature

	def erase(self):
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1) # set P3.2
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.top.cmdDelay(0.5)
		self.__setPx(P26=1, P33=1)
		self.top.cmdSetVPPVoltage(12)
		self.__runCommandSync(3)
		self.top.cmdDelay(0.5)
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(5)
		self.__setPx()
		self.__loadCommand(6) # VPP off
		self.top.flushCommands()
		self.top.printInfo("{chipid}: Erasing flash, verifying ...".format(chipid=self.chipDescription.chipID))
		ok = self.__verifyErase()
		if ok == 0:
			self.top.printInfo("{chipid}: Erase done.".format(chipid=self.chipDescription.chipID))
		else:
			self.top.printInfo("{chipid}: Erase failed!".format(chipid=self.chipDescription.chipID))

	def readProgmem(self):
		self.__initChip()
		self.top.cmdEnableZifPullups(True)
		self.applyGND(True)
		self.applyVCC(True)
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.__loadCommand(1) # set nPROG
		self.__setPx(P36=1, P37=1)
		#self.__setPx(P26=1, P27=1, P36=1)
		#self.__setPx()
		image = ""
		byteCount = 0
		self.progressMeterInit("Reading Flash", self.flashPageSize*self.flashPages)
		for addr in range(0, self.flashPageSize*self.flashPages):
			self.progressMeter(addr)
			self.__loadAddress(addr)
			#self.__runCommandSync(4)
			self.top.cmdFPGARead(0x10)
			byteCount += 1
			if byteCount == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(byteCount)
				byteCount = 0
		image += self.top.cmdReadBufferReg(byteCount)
		self.applyVPP(False)
		self.__setPx()
		self.__loadCommand(6) # VPP off
		self.top.flushCommands()
		self.progressMeterFinish()

		return image

	def writeProgmem(self, image):
		if len(image) > self.flashPageSize*self.flashPages:
			self.throwError("Invalid FLASH image size %d (expected <=%d)" %\
				(len(image), self.flashPageSize*self.flashPages))
		self.__initChip()
		self.top.cmdEnableZifPullups(True)
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1) # set P3.2
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.__setPx(P27=1, P33=1, P36=1, P37=1)
		self.top.cmdSetVPPVoltage(12)
		self.progressMeterInit("Writing Flash", len(image))
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			data = byte2int(image[addr])
			if data != 0xFF:
				self.__loadData(data)
				self.__loadAddress(addr)
				self.__loadCommand(3)
				ok = self.__progWait()
				if (ok & self.STAT_ERR) != 0:
					self.throwError("Write byte failed.")
		self.top.flushCommands()
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(5)
		self.__setPx()
		self.__loadCommand(6) # VPP off
		self.progressMeterFinish()
		ok = self.__verifyProgmem(image)
		if ok == 0:
			self.top.printInfo("{chipid}: Write flash done.".format(chipid = self.chipDescription.chipID))
		else:
			self.top.printInfo("{chipid}: Write flash failed!".format(chipid = self.chipDescription.chipID))
			
	def readLockbits(self):
		self.__initChip()
		self.top.cmdEnableZifPullups(True)		
		self.applyGND(True)
		self.applyVCC(True)
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.__loadCommand(1) # set nPROG
		self.__setPx(P26=1, P27=1, P36=1)
		data = ""
		self.top.cmdFPGARead(0x10)
		data += self.top.cmdReadBufferReg()
		self.applyVPP(False)
		self.__loadCommand(6) # VPP off
		lockbits = ""
		lockbits += data[0]
		return lockbits

	def writeLockbits(self, image):
		if len(image) != 1:
			self.throwError("Invalid lock-bits image size %d (expected %d)" %\
				(len(image), 1))
		self.progressMeterInit("Writing lock bits", 3)
		lbMask = 0x04
		lb=byte2int(image[0])
		if(lb & lbMask ):
			self.progressMeter(1)
			self.__writeLockbit(1)
		lbMask <<= 1
		if(lb & lbMask ):
			self.progressMeter(2)
			self.__writeLockbit(2)
		lbMask <<= 1
		if(lb & lbMask ):
			self.progressMeter(3)
			self.__writeLockbit(3)
		self.progressMeterFinish()
	
	def __writeLockbit(self, lb):
		self.__initChip()
		self.applyGND(True)
		self.applyVCC(True)
		self.__loadCommand(1)
		self.top.cmdSetVPPVoltage(5)
		self.__loadCommand(5) # VPP on
		self.applyVPP(True)
		self.top.cmdSetVPPVoltage(12)
		if(lb == 1 ):
			self.__setPx(P26=1, P27=1, P33=1, P36=1, P37=1)
			self.__runCommandSync(3)
		elif(lb == 2 ):
			self.__setPx(P26=1, P27=1, P33=1, P36=0, P37=0)
			self.__runCommandSync(3)
		elif(lb == 3 ):
			self.__setPx(P26=1, P27=0, P33=1, P36=1, P37=0)
			self.__runCommandSync(3)
		self.top.flushCommands()
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(5)
		self.__setPx()
		self.__loadCommand(6) # VPP off
		
	def __verifyErase(self):
		ok = 0
		image = self.readProgmem()
		for addr in range(0, self.flashPageSize*self.flashPages):
			if byte2int(image[addr]) != 0xFF:
				ok = 1
		return ok

	def __verifyProgmem(self,image):
		data = self.readProgmem()
		ok = 0
		for addr in range(0, len(image)-1):
			if byte2int(image[addr]) != byte2int(data[addr]):
				ok = 1
		return ok

	def __loadData(self, data):
		self.top.cmdFPGAWrite(0x10, data)
		
	def __loadAddress(self, addr):
		self.top.cmdFPGAWrite(0x11, addr & 0x00FF)
		self.top.cmdFPGAWrite(0x12, (addr >> 8) & 0x3FFF)		

	def __loadCommand(self, command):
		self.top.cmdFPGAWrite(0x13, command & 0xFF)

	def __runCommandSync(self, command):
		self.__loadCommand(command)
		self.__busyWait()

	def __setPx(self, P26=0, P27=0, P33=0, P36=0, P37=0, nPSEN=0, RST=1):
		data = 0
		if P26:
			data |= 1
		if P27:
			data |= 2
		if P33:
			data |= 4
		if P36:
			data |= 8
		if P37:
			data |= 16
		if nPSEN:
			data |= 32
		if RST:
			data |= 64
		self.top.cmdFPGAWrite(0x16, data)

	def __getStatusFlags(self):
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def __busy(self):
		return bool(self.__getStatusFlags() & self.STAT_BUSY)

	def __busyWait(self):
		for i in range(0, 26):
			if not self.__busy():
				return
			self.top.hostDelay(0.001)
		self.throwError("Timeout in busywait.")

	def __progWait(self):
		for i in range(0,4):
			self.top.cmdFPGARead(0x12)
			stat = self.top.cmdReadBufferReg()
			if (byte2int(stat[0]) & self.STAT_BUSY) == 0:
				return byte2int(stat[0])
			self.top.hostDelay(0.001)
		self.throwError("Timeout in busywait.")
lockbitDesc = (
	BitDescription(0, "Unused"),
	BitDescription(1, "Unused"),
	BitDescription(2, "BLB01 - further programming of flash mem. is disabled"),
	BitDescription(3, "BLB02 - verify disabled"),
	BitDescription(4, "BLB03 - external execution disabled"),
	BitDescription(5, "Unused"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
)
ChipDescription(
	Chip_AT89S52dip40,
	bitfile = "at89s5xdip40",
	chipID = "at89s52dip40",	
	runtimeID = (0x0005, 0x01),
	chipVendors = "Atmel",
	description = "AT89S52",
	lockbitDesc = lockbitDesc,
	maintainer = None,
	packages = ( ("DIP40", ""), )
)



"""
#    TOP2049 Open Source programming suite
#
#    ZIF socket layout generator
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *
from top_devices import *


class LayoutGenerator:
	"Layout generator baseclass."

	class MapError(Exception): pass

	def __init__(self):
		pass

	def setProgrammerType(self, programmer="TOP2049"):
		supportedDevices = {
			# Map  deviceName : layoutModules, ZIF-pin-count
			"TOP2049"	: (top2049.vcc_layouts, top2049.vpp_layouts,
					   top2049.gnd_layouts, 48)
		}
		try:
			(vcc_layouts, vpp_layouts, gnd_layouts, zifPins) = \
				supportedDevices[programmer.upper()]
		except (KeyError), e:
			raise TOPException("Programmer " + programmer + " not supported")
		self.vccLayout = vcc_layouts.VCCLayout()
		self.vppLayout = vpp_layouts.VPPLayout()
		self.gndLayout = gnd_layouts.GNDLayout()
		self.zifPins = zifPins

	def setPins(self, vccPin, vppPins, gndPin):
		"""Load the supply pin locations.
		vppPins may either be one pin number or a list of pin numbers or None."""
		self.vccPin = vccPin
		if vppPins is None:
			self.vppPins = [ ]
		else:
			try:
				self.vppPins = list(vppPins)
			except TypeError:
				self.vppPins = [ vppPins, ]
		self.gndPin = gndPin
		self.verifyPins()

	def getZifPinForPackagePin(self, packagePin):
		"Get the ZIF pin number corresponding to the package pin number"
		return self.mapPin2zif(packagePin, self.result_offset,
				       self.result_upsideDown)

	def verifyPins(self):
		pass

	def maxOffset(self, upsideDown):
		"Returns the max possible chip offset (in the ZIF socket)"
		raise TOPException("Reimplement me")

	def mapToZIF(self, offset, upsideDown):
		"Tries to map the chip into the ZIF socket"
		if offset < 0 or offset > self.maxOffset(upsideDown):
			raise self.MapError()

		# Find a GND layout
		zifGndPin = self.mapPin2zif(self.gndPin, offset, upsideDown)
		self.result_GND = self.__findSingleLayout(
					self.gndLayout,
					(1 << (zifGndPin - 1)))

		# Find a VCC layout
		zifVccPin = self.mapPin2zif(self.vccPin, offset, upsideDown)
		self.result_VCC = self.__findSingleLayout(
					self.vccLayout,
					(1 << (zifVccPin - 1)))

		# Find a (possibly cumulative) VPP layout
		if not self.vppPins:
			self.result_VPP = None
		else:
			zifVppMask = 0
			for vppPin in self.vppPins:
				pin = self.mapPin2zif(vppPin, offset, upsideDown)
				zifVppMask |= (1 << (pin - 1))
			self.result_VPP = self.__findCumulativeLayout(
						self.vppLayout,
						zifVppMask)

		# Also store the chip orientation for later use.
		self.result_upsideDown = upsideDown
		self.result_offset = offset

	def mapPin2zif(self, packagePin, offset, upsideDown):
		"Map a package pin to a ZIF pin. Returns the ZIF pin number."
		raise TOPException("Reimplement me")

	def zifLayoutAsciiArt(self):
		"Returns nice ascii ART of the mapped ZIF socket"
		raise TOPException("Reimplement me")

	def zifPinAssignments(self):
		"Returns a string describing the pin assignments"
		vcc = str(self.__bitmask2pinList(self.result_VCC[1])).strip("[]")
		ret =  "VCC ZIF pins: " + vcc + "\n"
		if self.result_VPP:
			vppBitmask = 0
			for (id, mask) in self.result_VPP:
				vppBitmask |= mask
			vpp = str(self.__bitmask2pinList(vppBitmask)).strip("[]")
			ret += "VPP ZIF pins:  " + vpp + "\n"
		gnd = str(self.__bitmask2pinList(self.result_GND[1])).strip("[]")
		ret += "GND ZIF pins:  " + gnd + "\n"
		return ret

	def __bitmask2pinList(self, bitmask):
		ret = []
		bit = 0
		while bitmask:
			if bitmask & (1 << bit):
				ret.append(bit + 1)
			bitmask &= ~(1 << bit)
			bit += 1
		return ret

	def __pinList2Bitmask(self, pinList):
		bitmask = 0
		for pin in pinList:
			assert(pin >= 1)
			bitmask |= (1 << (pin - 1))
		return bitmask

	def recalculate(self):
		"Redo the mapping calculation"
		for upsideDown in (False, True):
			offset = self.maxOffset(upsideDown)
			while offset >= 0:
				try:
					self.mapToZIF(offset, upsideDown)
				except (LayoutGenerator.MapError), e:
					offset -= 1
					continue
				return
		raise TOPException("Did not find a possible valid layout for the setup")

	def getGNDLayout(self):
		"Get the calculated GND layout ID and mask. Returns a tuple (ID, mask)."
		return self.result_GND

	def applyGNDLayout(self, top):
		"Send the GND layout to hardware"
		(layoutID, layoutMask) = self.getGNDLayout()
		top.gnd.setLayoutID(layoutID)

	def getVCCLayout(self):
		"Get the calculated VCC layout ID and mask. Returns a tuple (ID, mask)."
		return self.result_VCC

	def applyVCCLayout(self, top):
		"Send the VCC layout to hardware"
		(layoutID, layoutMask) = self.getVCCLayout()
		top.vcc.setLayoutID(layoutID)

	def getVPPLayouts(self):
		"""Get the calculated VPP layout IDs and masks.
		Returns a list of tuples ((ID, mask), (ID, mask), ...)"""
		return self.result_VPP

	def applyVPPLayout(self, top, packagePins=[]):
		"""Send the VPP layout to hardware.
		packagePins is a list of pins (on the chip package) to activate.
		If packagePins is not passed, all VPP pins are driven to VPP."""
		if packagePins:
			pins = []
			for pin in packagePins:
				pins.append(self.mapPin2zif(pin, self.result_offset,
							    self.result_upsideDown))
			packagePinsMask = self.__pinList2Bitmask(pins)
		top.vpp.setLayoutMask(0) # Reset
		layouts = self.getVPPLayouts()
		if layouts:
			for (layoutID, mask) in layouts:
				if packagePins:
					if mask & packagePinsMask == 0:
						continue
					if mask & packagePinsMask != mask:
						raise TOPException(
							"Unable to apply partial VPP layout")
				top.vpp.setLayoutID(layoutID)

	def __findSingleLayout(self, layoutDefs, zifBitmask):
		# Returns an (id, mask) tuple
		for (id, mask) in layoutDefs.supportedLayouts():
			if zifBitmask == mask:
				break
		else:
			raise self.MapError()
		return (id, mask)

	def __findCumulativeLayout(self, layoutDefs, zifBitmask):
		# Returns a list of (id, mask) tuples
		result = []
		for (id, mask) in layoutDefs.supportedLayouts():
			if zifBitmask == 0:
				break
			if mask == 0:
				continue
			if mask & zifBitmask == mask:
				result.append( (id, mask) )
				zifBitmask &= ~mask
		if zifBitmask:
			raise self.MapError()
		return result

class LayoutGeneratorDIP(LayoutGenerator):
	"Layout generator for DIP packages."

	def __init__(self, nrPins):
		LayoutGenerator.__init__(self)
		self.nrPins = nrPins

	def verifyPins(self):
		LayoutGenerator.verifyPins(self)
		if self.nrPins < 2 or self.nrPins > self.zifPins or self.nrPins % 2 != 0:
			raise TOPException("Invalid DIP package")
		if self.vccPin < 1 or self.vccPin > self.nrPins:
			raise TOPException("Invalid VCC pin number for the selected package")
		for vppPin in self.vppPins:
			if vppPin < 1 or vppPin > self.nrPins:
				raise TOPException("Invalid VPP pin number for the selected package")
		if self.gndPin < 1 or self.gndPin > self.nrPins:
			raise TOPException("Invalid GND pin number for the selected package")

	def maxOffset(self, upsideDown):
		return self.zifPins // 2 - self.nrPins // 2

	def mapPin2zif(self, dipPin, offset, upsideDown):
		assert(dipPin >= 1 and dipPin <= self.nrPins)
		if upsideDown:
			if dipPin > self.nrPins // 2:
				# Right side of DIP
				dipPin -= self.nrPins // 2
				return dipPin + offset
			else:
				# Left side of DIP
				return self.zifPins - self.nrPins // 2 + dipPin - offset
		else:
			if dipPin > self.nrPins // 2:
				# Right side of DIP
				dipPin -= self.nrPins // 2
				return self.zifPins - self.nrPins // 2 + dipPin - offset
			else:
				# Left side of DIP
				return dipPin + offset

	def zifLayoutAsciiArt(self):
		def line(prefix, content, postfix):
			return "%3s  %s  %3s\n" % (prefix, content, postfix)

		zifGnd = self.getZifPinForPackagePin(self.gndPin)
		zifVcc = self.getZifPinForPackagePin(self.vccPin)
		zifVpp = [ self.getZifPinForPackagePin(p) \
			   for p in self.vppPins ]

		ret =  line("", "T     ZIF socket", "")
		ret += line("", "^--o==============o", "")
		for zp in range(1, self.zifPins // 2 + 1):
			prefix, postfix = "", ""
			if zp == zifGnd:
				prefix = "GND"
			if self.zifPins - zp + 1 == zifGnd:
				postfix = "GND"
			if zp == zifVcc:
				prefix = "VCC"
			if self.zifPins - zp + 1 == zifVcc:
				postfix = "VCC"
			if zp in zifVpp:
				prefix = "VPP"
			if self.zifPins - zp + 1 in zifVpp:
				postfix = "VPP"

			if zp < self.result_offset + 1 or \
			   zp > self.result_offset + self.nrPins // 2:
				ret += line(prefix,
					"%2d |----- || -----| %2d" %\
					(zp, self.zifPins + 1 - zp),
					postfix)
			else:
				if zp == self.result_offset + 1 and \
				   not self.result_upsideDown:
					ret += line(prefix,
						"%2d |-- 1##..##o --| %2d" %\
						(zp, self.zifPins + 1 - zp),
						postfix)
				elif zp == self.result_offset + self.nrPins // 2 and \
				     self.result_upsideDown:
					ret += line(prefix,
						"%2d |-- o##''##1 --| %2d" %\
						(zp, self.zifPins + 1 - zp),
						postfix)
				else:
					ret += line(prefix,
						"%2d |-- o######o --| %2d" %\
						(zp, self.zifPins + 1 - zp),
						postfix)
		ret += line("", "   o==============o", "")
		return ret

def createLayoutGenerator(package):
	try:
		for regex in ("DIP(\d+)", "PDIP(\d+)", "SO(\d+)", "TSSOP(\d+)", ):
			m = re.match(regex, package, re.IGNORECASE)
			if m:
				nrPins = int(m.group(1))
				return LayoutGeneratorDIP(nrPins)
		if package.upper() == "PLCC32": # 1:1 adapter
			return LayoutGeneratorDIP(32)
		if package.upper() == "PLCC44": # 1:1 adapter
			return LayoutGeneratorDIP(44)
		raise ValueError()
	except (ValueError), e:
		raise TOPException("Unknown package type " + package)



#!/usr/bin/env python

"""
#    TOP2049 Open Source programming suite
#
#    Commandline utility
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import re
import sys
import os

def substitute(input, oldSocket, newSocket):
	input = re.sub('(^\s*packages).*', lambda m:'{} = (("DIP10", ""), ),'.format(m.group(1)), input)
	input = re.sub('(^\s*chipPackage).*', lambda m:'{} = "DIP10",'.format(m.group(1)), input)
	input = re.sub('(^\s*chipPinVCC).*', lambda m:'{} = 9,'.format(m.group(1)), input)
	input = re.sub('(^\s*chipPinsVPP).*', lambda m:'{} = 10,'.format(m.group(1)), input)
	input = re.sub('(^\s*chipPinGND).*', lambda m:'{} = 8,'.format(m.group(1)), input)
	input = re.sub('(^\s*runtimeID).*', lambda m:'{} = (0xDE05, 0x01),'.format(m.group(1)), input)
	input = re.sub('(^\s*description).+"(.*)".*', lambda m:'{} = "{} - ICD",'.format(m.group(1), m.group(2)), input)
	input = re.sub('(^\s*bitfile).*', lambda m:'{} = "microchip01sip6",'.format(m.group(1)), input)
	input = re.sub("{}".format(oldSocket), "{}".format(newSocket), input)
	input = re.sub("{}".format(oldSocket.upper()), "{}".format(newSocket.upper()), input)
	return input

def makeSip():
	inputFileName = '__init__.py'
	fin = open(inputFileName)
	dMCU = {}
	for line in fin:
		matchObj = re.match('.*(pic[0-9]+l?f\w+)(sip[0-9a]+).*', line)
		if matchObj:
			continue
		matchObj = re.match('.*(pic[0-9]+l?f\w+)(dip[0-9a]+).*', line)
		if not matchObj:
			print("{} did not match".format(line))
			continue
#		print('matched {} - {}'.format(matchObj.group(1), matchObj.group(2)))
		dMCU.setdefault(matchObj.group(1), matchObj.group(2))
	fin.close()
	for item in dMCU.items():
		fin = open("{}{}.py".format(item[0], item[1]))
		fout = open("{}sip6.py".format(item[0]), 'w')
		fout.write("#\n")
		fout.write("# THIS FILE WAS AUTOGENERATED BY makeSip6.py\n")
		fout.write("# Do not edit this file manually. All changes will be lost.\n")
		fout.write("#\n\n")
		for line in fin:
			fout.write(substitute(line, "{}".format(item[1]), "sip6"))
		fout.close()
		fin.close()

def main(argv):
	makeSip()

if __name__ == "__main__":
	exit(main(sys.argv))



"""
#    TOP2049 Open Source programming suite
#
#    Generic programming algorithms
#
#    Copyright (c) 2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *


class AddrSetter(object):
	"""Generic address setter"""

	def __init__(self, chip,
		     fpgaCmdByte0,
		     fpgaCmdByte1 = -1,
		     fpgaCmdByte2 = -1,
		     fpgaCmdByte3 = -1,
		     addrSetupFunc = lambda byteNr, data: None,
		     addrFinishFunc = lambda byteNr, data: None):
		"""
		chip => The Chip.
		fpgaCmdByte0 => FPGA command number for setting addr byte 0.
		fpgaCmdByte1 => FPGA command number for setting addr byte 1.
		fpgaCmdByte2 => FPGA command number for setting addr byte 2.
		fpgaCmdByte3 => FPGA command number for setting addr byte 3.
		addrSetupFunc => Optional function called before addr write.
		addrFinishFunc => Optional function called after addr write.
		"""
		self.chip = chip
		self.fpgaCmds = [
			fpgaCmdByte0,
			fpgaCmdByte1,
			fpgaCmdByte2,
			fpgaCmdByte3,
		]
		self.addrSetupFunc = addrSetupFunc
		self.addrFinishFunc = addrFinishFunc
		self.reset()

	def reset(self):
		self.prevAddr = None

	def load(self, addr):
		"""Load an address into the FPGA.
		This loads only the bytes that changed."""
		shifts = (0, 8, 16, 24)
		for byteNr, shift in enumerate(shifts):
			fpgaCmd = self.fpgaCmds[byteNr]
			if fpgaCmd < 0:
				continue
			addrByte = (addr >> shift) & 0xFF
			if self.prevAddr is None or\
			   ((self.prevAddr >> shift) & 0xFF) != addrByte:
				self.addrSetupFunc(byteNr, addrByte)
				self.chip.top.cmdFPGAWrite(fpgaCmd, addrByte)
				self.addrFinishFunc(byteNr, addrByte)
		self.prevAddr = addr

class GenericAlgorithms(object):
	"""Generic programming algorithms."""

	def __init__(self, chip):
		self.chip = chip

	def simpleVoltageSetup(self, vcc=5.0, vpp=5.0, vppEnable=False):
		"""Simple voltage setup."""
		self.chip.top.cmdSetVCCVoltage(vcc)
		self.chip.top.cmdSetVPPVoltage(vpp)
		self.chip.applyVCC(True)
		self.chip.applyGND(True)
		self.chip.applyVPP(vppEnable)

	def simpleVoltageShutdown(self):
		"""Turn off voltages."""
		self.chip.applyVCC(False)
		self.chip.applyVPP(False)
		self.chip.applyGND(False)
		self.chip.top.cmdSetVCCVoltage(self.chip.top.vcc.minVoltage())
		self.chip.top.cmdSetVPPVoltage(self.chip.top.vpp.minVoltage())

	def simpleRead(self, name, sizeBytes,
		       readData8Func,
		       addrSetter,
		       initFunc = lambda: None,
		       exitFunc = lambda: None):
		"""Simple 8-bit data read algorithm."""
		self.chip.progressMeterInit("Reading %s" % name, sizeBytes)
		image, count = [], 0
		initFunc()
		addrSetter.reset()
		for addr in range(0, sizeBytes):
			self.chip.progressMeter(addr)
			addrSetter.load(addr)
			readData8Func()
			count += 1
			if count == self.chip.top.getBufferRegSize():
				image.append(self.chip.top.cmdReadBufferReg())
				count = 0
		if count:
			image.append(self.chip.top.cmdReadBufferReg()[:count])
		exitFunc()
		self.chip.progressMeterFinish()
		return b"".join(image)

	def simpleReadEPROM(self, sizeBytes,
			    readData8Func,
			    addrSetter,
			    initFunc = lambda: None,
			    exitFunc = lambda: None):
		return self.simpleRead("EPROM", sizeBytes, readData8Func,
				       addrSetter, initFunc, exitFunc)

	def simpleReadEEPROM(self, sizeBytes,
			     readData8Func,
			     addrSetter,
			     initFunc = lambda: None,
			     exitFunc = lambda: None):
		return self.simpleRead("EEPROM", sizeBytes, readData8Func,
				       addrSetter, initFunc, exitFunc)

	def simpleTest(self, readFunc, writeFunc, size):
		"""Simple Unit-test."""
		image = genRandomBlob(size)
		writeFunc(image)
		if readFunc() != image:
			self.chip.throwError("Unit-test failed. "
				"The chip may be physically broken.")



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC16F84 DIP18
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_splittedPMarea import *

class Chip_Pic16F84adip18(microchip8_splittedPMarea):
	CMD_BEGIN_ERASE_PROGRAMMING_CYCLE = 0x8
	CMD_BEGIN_PROGRAMMING_ONLY_CYCLE = 0x18
	
	voltageVDD = 5
	voltageVPP = 13
	
	delayTinternalProgPM = 0.004

	userIDLocationSize = 4
	hasEEPROM = True

	def __init__(self):
	 	microchip8_splittedPMarea.__init__(self,
		chipPackage="DIP18",
		chipPinVCC=14,
		chipPinsVPP=4,
		chipPinGND=5,
		signature="\x60\x05",
		flashPageSize=0x200,
		flashPages=1,
		eepromPageSize=64,
		eepromPages=1,
		fuseBytes=2
		)
		
	def sendWriteFlashInstr(self):
		'''
		'''
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_PROGRAMMING_ONLY_CYCLE)
		self.top.cmdDelay(self.delayTinternalProgDM)
		
	def sendWriteFlashInstrDM(self):
		self.sendWriteFlashInstr()
	
	def bulkErasePGM(self):	
		self.sendCommand(0, 0, 0, self.CMD_BULK_ERASE_PGM)
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_PROGRAMMING_ONLY_CYCLE)
		self.top.cmdDelay(self.delayTera)  # Tera
		
	def bulkEraseDM(self):
		self.sendCommand(0, 0, 0, self.CMD_BULK_ERASE_DM)
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_PROGRAMMING_ONLY_CYCLE)
		self.top.cmdDelay(self.delayTera)  # Tera		
fuseDesc = (
	BitDescription(0, "FOSC[0], 00=LP osc, 01=XT osc"),
	BitDescription(1, "FOSC[1], 10=HS osc, 11=RC osc"),
	BitDescription(2, "WDTEN, 1=WDT enabled"),
	BitDescription(3, "nPWRT"),
	BitDescription(4, "nCP"),
	BitDescription(5, "nCP"),
	BitDescription(6, "nCP"),
	BitDescription(7, "nCP"),
	BitDescription(8, "nCP"),
	BitDescription(9, "nCP"),
	BitDescription(10, "nCP"),
	BitDescription(11, "nCP"),
	BitDescription(12, "nCP"),
	BitDescription(13, "nCP"),
)

ChipDescription(
	Chip_Pic16F84adip18,
	bitfile="microchip01dip18",
	chipID="pic16f84adip18",
	runtimeID=(0xDE04, 0x01),
	chipVendors="Microchip",
	description="PIC16F84A, PIC16LF84A",
	packages=(("DIP18", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    Generic user interface support
#
#    Copyright (c) 2009-2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys


class AbstractUserInterface:
	"Abstract user interface class. Subclass it."

	# Progress meter IDs
	PROGRESSMETER_CHIPACCESS	= 0 # Programming/reading progress meter
	PROGRESSMETER_USER		= 100 # IDs equal or bigger than this are available to the app

	def progressMeterInit(self, meterId, message, nrSteps):
		pass # Do nothing by default

	def progressMeterFinish(self, meterId):
		pass # Do nothing by default

	def progressMeter(self, meterId, step):
		pass # Do nothing by default

	def warningMessage(self, message):
		pass # Do nothing by default

	def infoMessage(self, message):
		pass # Do nothing by default

	def debugMessage(self, message):
		pass # Do nothing by default

class ConsoleUserInterface(AbstractUserInterface):
	"Console based user interface."

	def progressMeterInit(self, meterId, message, nrSteps):
		if meterId != self.PROGRESSMETER_CHIPACCESS:
			return
		self.progressNrSteps = nrSteps
		self.progress = range(0, 100)
		self.__genericConsoleMessage(message + " [0%", newline=False)

	def progressMeterFinish(self, meterId):
		if meterId != self.PROGRESSMETER_CHIPACCESS:
			return
		if not self.progressNrSteps:
			self.progressMeter(meterId, 0)
		self.__genericConsoleMessage("100%]")

	def progressMeter(self, meterId, step):
		if meterId != self.PROGRESSMETER_CHIPACCESS:
			return
		if self.progressNrSteps:
			percent = (step * 100 // self.progressNrSteps)
		else:
			percent = 100
		for progress in list(self.progress):
			if progress > percent:
				break
			if progress == 25:
				self.__genericConsoleMessage("25%", newline=False)
			elif progress == 50:
				self.__genericConsoleMessage("50%", newline=False)
			elif progress == 75:
				self.__genericConsoleMessage("75%", newline=False)
			elif progress % 2 == 0:
				self.__genericConsoleMessage(".", newline=False)
			self.progress.remove(progress)

	def __genericConsoleMessage(self, message, newline=True):
		if newline:
			print message
		else:
			sys.stdout.write(message)
			sys.stdout.flush()

	def warningMessage(self, message):
		self.__genericConsoleMessage(message)

	def infoMessage(self, message):
		self.__genericConsoleMessage(message)

	def debugMessage(self, message):
		self.__genericConsoleMessage(message)



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8 - 18f2221 family - 8bit PIC MCU
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_18_common import *

class microchip8_18f2221family(Chip_Microchip8_18_common):
	
	voltageVDD				 = 5
	voltageVPP				 = 11
	
	delayP2A = 0.0000008  # Serial clock low time 
  	delayP5 = 0.0000015  # Delay between 4-bit command and command operand
  	
	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip_Microchip8_18_common.__init__(self,
		chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes)	
		
	def erase(self):
		self.progressMeterInit("Erasing chip", 0)
		self.enterPM(True)
		self.executeCode(self.getCodeAddrToTBLPTR(0x3C0005))
		self.send4bitWriteInstruction(self.CMD_TW, 0x3F3F)
		self.executeCode(self.getCodeAddrToTBLPTR(0x3C0004))
		self.send4bitWriteInstruction(self.CMD_TW, 0x8F8F)
		self.executeCode((0x0000,))
		self.sendCommand(1)		
		self.top.hostDelay(self.delayP11 + self.delayP10)
		for i in range(0,4):
			self.sendCommand(1)
		self.top.flushCommands()
		self.progressMeterFinish()
	
	def eraseRow(self, rowAddr):
		self.executeCode((0x8EA6, 0x9CA6, 0x84A6))
		self.executeCode(self.getCodeAddrToTBLPTR(rowAddr))
		self.executeCode((0x88A6, 0x82A6))
		self.sendCommand(1, 0, 0, 0x0, 1)
		self.top.hostDelay(self.delayP9)
		self.setPins(0)
		self.top.hostDelay(self.delayP10)	

	def setEEPROMAddr(self, addr):
		self.executeCode((0x0E00 | (addr & 0xFF), 0x6EA9, (0x0E00 | ((addr >> 8) & 0xFF)), 0x6EAA))
			



"""
#    TOP2049 Open Source programming suite
#
#    Universal device tester
#
#    Copyright (c) 2010-2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_Unitest(Chip):
	MODE_UNITEST	= 0	# Standard universal tester
	MODE_FCNT	= 1	# Frequency counter

	def __init__(self, chipPackage=None, chipPinVCC=None, chipPinsVPP=None, chipPinGND=None,
			   VCCVoltage=None, VPPVoltage=None):
		Chip.__init__(self, chipPackage=chipPackage, chipPinVCC=chipPinVCC,
			      chipPinsVPP=chipPinsVPP, chipPinGND=chipPinGND)
		self.autogenVCCVoltage = VCCVoltage
		self.autogenVPPVoltage = VPPVoltage
		self.mode = self.MODE_UNITEST

	def shutdownChip(self):
		self.printDebug("Shutdown chip")
		self.reset()

	def reset(self):
		self.top.vcc.setLayoutPins( [] )
		self.vccMask = 0
		self.top.vpp.setLayoutPins( [] )
		self.vppMask = 0
		self.top.gnd.setLayoutPins( [] )
		self.gndMask = 0
		self.top.cmdSetVCCVoltage(self.top.vcc.minVoltage())
		self.top.cmdSetVPPVoltage(self.top.vpp.minVoltage())
		self.oscMask = 0
		self.oscDiv = 0
		self.fcntPin = 0
		self.setOutputEnableMask(0)
		self.setOutputs(0)
		self.setOscMask(0)
		self.top.flushCommands()

	def getMode(self):
		return self.mode

	def setMode(self, newMode):
		if self.mode == newMode:
			return
		# Upload the new bitfile to the FPGA
		bitfiles = {
			self.MODE_UNITEST	: ("unitest", (8, 1)),
			self.MODE_FCNT		: ("unitest_fcnt", (8, 2)),
		}
		name, runtimeIDs = bitfiles[newMode]
		self.top.configureFPGA(name, runtimeIDs)
		# Re-upload the FPGA state
		self.__updateOutEn()
		self.__updateOut()
		self.__updateOscDivider()
		self.__updateOscMask()
		self.__updateFreqCountPin()
		self.top.flushCommands()
		self.mode = newMode

	def setVCC(self, voltage, layout):
		self.vccMask = self.top.vcc.ID2mask(layout)
		self.__updateOutEn()
		self.top.cmdSetVCCVoltage(voltage)
		self.top.vcc.setLayoutID(layout)
		self.top.flushCommands()

	def setVPP(self, voltage, layouts):
		self.vppMask = 0
		for layout in layouts:
			self.vppMask |= self.top.vpp.ID2mask(layout)
		self.__updateOutEn()
		self.top.cmdSetVPPVoltage(voltage)
		self.top.vpp.setLayoutMask(0) # Reset
		for layout in layouts:
			self.top.vpp.setLayoutID(layout)
		self.top.flushCommands()

	def setGND(self, layout):
		self.gndMask = self.top.gnd.ID2mask(layout)
		self.__updateOutEn()
		self.top.gnd.setLayoutID(layout)
		self.top.flushCommands()

	# Overloaded layout generator interface.
	def applyVCC(self, turnOn):
		layoutID = 0
		if turnOn:
			(layoutID, layoutMask) = self.generator.getVCCLayout()
		self.setVCC(self.autogenVCCVoltage, layoutID)

	# Overloaded layout generator interface.
	def applyVPP(self, turnOn, packagePinsToTurnOn=[]):
		assert(not packagePinsToTurnOn) # Not supported, yet.
		layouts = []
		if turnOn:
			layouts = map(lambda (layoutID, layoutMask): layoutID,
				      self.generator.getVPPLayouts())
		self.setVPP(self.autogenVPPVoltage, layouts)

	# Overloaded layout generator interface.
	def applyGND(self, turnOn):
		layoutID = 0
		if turnOn:
			(layoutID, layoutMask) = self.generator.getGNDLayout()
		self.setGND(layoutID)

	def getActivePinMask(self):
		masks = {
			self.MODE_UNITEST	: 0xFFFFFFFFFFFF, # 1-48
			self.MODE_FCNT		: 0x000FFFFFF000, # 13-36
		}
		return masks[self.mode]

	def __updateOutEn(self):
		mask = self.desiredOutEnMask
		mask &= ~self.gndMask
		mask &= ~self.vccMask
		mask &= ~self.vppMask
		mask |= self.oscMask
		mask &= self.getActivePinMask()
		self.top.cmdFPGAWrite(0x40, mask & 0xFF)
		self.top.cmdFPGAWrite(0x41, (mask >> 8) & 0xFF)
		self.top.cmdFPGAWrite(0x42, (mask >> 16) & 0xFF)
		self.top.cmdFPGAWrite(0x43, (mask >> 24) & 0xFF)
		self.top.cmdFPGAWrite(0x44, (mask >> 32) & 0xFF)
		self.top.cmdFPGAWrite(0x45, (mask >> 40) & 0xFF)

	def setOutputEnableMask(self, mask):
		self.desiredOutEnMask = mask
		self.__updateOutEn()
		self.top.flushCommands()

	def __updateOut(self):
		mask = self.desiredOutMask
		mask &= ~self.oscMask
		mask &= self.getActivePinMask()
		self.top.cmdFPGAWrite(0x60, mask & 0xFF)
		self.top.cmdFPGAWrite(0x61, (mask >> 8) & 0xFF)
		self.top.cmdFPGAWrite(0x62, (mask >> 16) & 0xFF)
		self.top.cmdFPGAWrite(0x63, (mask >> 24) & 0xFF)
		self.top.cmdFPGAWrite(0x64, (mask >> 32) & 0xFF)
		self.top.cmdFPGAWrite(0x65, (mask >> 40) & 0xFF)

	def setOutputs(self, mask):
		self.desiredOutMask = mask
		self.__updateOut()
		self.top.flushCommands()

	def getInputs(self):
		self.top.cmdFPGARead(0x60)
		self.top.cmdFPGARead(0x61)
		self.top.cmdFPGARead(0x62)
		self.top.cmdFPGARead(0x63)
		self.top.cmdFPGARead(0x64)
		self.top.cmdFPGARead(0x65)
		inputs = self.top.cmdReadBufferReg48()
		return inputs

	def __updateOscDivider(self):
		div = self.oscDiv
		self.top.cmdFPGAWrite(0x00, div & 0xFF)
		self.top.cmdFPGAWrite(0x01, (div >> 8) & 0xFF)
		self.top.cmdFPGAWrite(0x02, (div >> 16) & 0xFF)
		self.top.cmdFPGAWrite(0x03, (div >> 24) & 0xFF)

	def setOscDivider(self, div):
		self.oscDiv = div
		self.__updateOscDivider()
		self.top.flushCommands()

	def __updateOscMask(self):
		mask = self.oscMask
		mask &= self.getActivePinMask()
		self.top.cmdFPGAWrite(0x20, mask & 0xFF)
		self.top.cmdFPGAWrite(0x21, (mask >> 8) & 0xFF)
		self.top.cmdFPGAWrite(0x22, (mask >> 16) & 0xFF)
		self.top.cmdFPGAWrite(0x23, (mask >> 24) & 0xFF)
		self.top.cmdFPGAWrite(0x24, (mask >> 32) & 0xFF)
		self.top.cmdFPGAWrite(0x25, (mask >> 40) & 0xFF)

	def setOscMask(self, mask):
		self.oscMask = mask
		self.__updateOscMask()
		self.__updateOutEn()
		self.__updateOut()
		self.top.flushCommands()

	def __updateFreqCountPin(self):
		if self.mode == self.MODE_FCNT:
			value = self.fcntPin
			self.top.cmdFPGAWrite(0x80, value)

	def setFreqCountPin(self, pinNumber, invert=False):
		assert(pinNumber <= 0x3F)
		value = pinNumber & 0x3F
		if invert:
			value |= 0x80
		self.fcntPin = value
		self.__updateFreqCountPin()
		self.top.flushCommands()

	def getFreqCount(self):
		if self.mode != self.MODE_FCNT:
			return 0
		self.top.cmdFPGARead(0x00)
		self.top.cmdFPGARead(0x01)
		self.top.cmdFPGARead(0x02)
		self.top.cmdFPGARead(0x03)
		count = self.top.cmdReadBufferReg32()
		return count

ChipDescription(
	Chip_Unitest,
	chipID = "unitest",
	bitfile = "unitest",
	runtimeID = (8, 1),
	chipType = ChipDescription.TYPE_INTERNAL,
	description = "Universal device tester",
)



"""
#    TOP2049 Open Source programming suite
#
#    Generic ZIF socket voltage layout
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.util import *


class GenericLayout:
	def __init__(self, nrZifPins):
		self.nrZifPins = nrZifPins

	def getNrOfPins(self):
		"Returns the number of pins on the layout"
		return self.nrZifPins

	def __repr__(self):
		res = ""
		for (id, zif_mask) in self.supportedLayouts():
			res += "Layout %d:\n" % id
			res += "        o---------o\n"
			for pin in range(1, self.nrZifPins // 2 + 1):
				left = "     "
				right = ""
				if (1 << (pin - 1)) & zif_mask:
					left = "HOT >"
				if (1 << (self.nrZifPins - pin)) & zif_mask:
					right = "< HOT"
				res += "%s   | %2d | %2d |   %s\n" % \
					(left, pin, self.nrZifPins + 1 - pin, right)
			res += "        o---------o\n\n"
		return res

	def supportedLayouts(self):
		"""Returns a list of supported layouts.
		Each entry is a tuple of (id, bitmask), where bitmask is
		the ZIF layout. bit0 is ZIF-pin-1. A bit set means a hot pin."""
		# Reimplement me in the subclass
		raise Exception()

	def ID2mask(self, id):
		"Convert a layout ID to a layout mask"
		for (layoutId, layoutMask) in self.supportedLayouts():
			if id == layoutId:
				return layoutMask
		return None

	def ID2pinlist(self, id):
		"Convert a layout ID to a list of pins"
		pinlist = []
		mask = self.ID2mask(id)
		for i in range(0, self.nrZifPins):
			if mask & (1 << i):
				pinlist.append(i + 1)
		return pinlist

	def setLayoutPins(self, zifPinsList):
		"""Load a layout. zifPinsList is a list of hot ZIF pins.
		The first ZIF pin is 1."""
		zifMask = 0
		for zifPin in zifPinsList:
			assert(zifPin >= 1)
			zifMask |= (1 << (zifPin - 1))
		self.setLayoutMask(zifMask)

	def setLayoutMask(self, zifMask):
		"Load a ZIF mask."
		for (layoutId, layoutMask) in self.supportedLayouts():
			if layoutMask == zifMask:
				self.setLayoutID(layoutId)
				return
		raise TOPException("Layout mask impossible due to hardware constraints")

	def setLayoutID(self, id):
		"Load a specific layout ID."
		# Reimplement me in the subclass
		raise Exception()



"""
#    TOP2049 Open Source programming suite
#
#    TOP2049 Shiftregister based layout definitions
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.generic_layout import *


class ShiftregLayout(GenericLayout):
	def __init__(self, nrZifPins, nrShiftRegs):
		GenericLayout.__init__(self, nrZifPins)
		self.nrShiftRegs = nrShiftRegs
		self.layouts = []
		for id in range(0, len(self.shiftreg_masks)):
			shreg_mask = self.shiftreg_masks[id]
			zif_mask = 0
			for bit in range(0, self.nrShiftRegs * 8):
				if (shreg_mask & (1 << bit)) == 0:
					continue
				regId = self.__bitnr2shregId(bit)
				zifPin = self.shreg2zif_map[regId]
				zif_mask |= (1 << (zifPin - 1))
			if id == 0 or zif_mask != 0:
				self.layouts.append( (id, zif_mask) )

	def __bitnr2shregId(self, bitNr):
		register = bitNr // 8
		pin = bitNr % 8
		return "%d.%d" % (register, pin)

	def supportedLayouts(self):
		return self.layouts



from pic24f04ka200dip14 import *
from pic24f04ka201dip20 import *
from pic24f16kl402dip28 import *
from pic24f08kl402dip28 import *
from pic24f16kl401dip20 import *
from pic24f08kl401dip20 import *
from pic24f08kl302dip28 import *
from pic24f08kl301dip20 import *
from pic24f08kl201dip20 import *
from pic24f08kl200dip14 import *
from pic24f04kl101dip20 import *
from pic24f04kl100dip14 import *
from pic24f04ka200sip6 import *
from pic24f04ka201sip6 import *
from pic24f04kl100sip6 import *
from pic24f04kl101sip6 import *
from pic24f08kl200sip6 import *
from pic24f08kl201sip6 import *
from pic24f08kl301sip6 import *
from pic24f08kl302sip6 import *
from pic24f08kl401sip6 import *
from pic24f08kl402sip6 import *
from pic24f16kl401sip6 import *
from pic24f16kl402sip6 import *



"""
#    TOP2049 Open Source programming suite
#
#    pic8_singlePMarea - file for older 8bit PIC MCUs
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_common import *

class microchip8_singlePMarea(Chip_Microchip8_common):
	CMD_BEGIN_PROGRAMMING = 0x08
	CMD_END_PROGRAMMING = 0x0E

	userIDLocationSize = 4
	hasSigBytes = False

	voltageVDD = 5
	voltageVPP = 13
	defaultWord = [b'\xFF', b'\x0F']

	def __init__(self,
	    chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
	    signature,
	    flashPageSize, flashPages,
	    eepromPageSize, eepromPages,
	    fuseBytes
	    ):
		Chip_Microchip8_common.__init__(self, chipPackage, chipPinVCC, chipPinsVPP, chipPinGND, signature, flashPageSize, flashPages, eepromPageSize, eepromPages, fuseBytes)
		self.initPcValue		 = self.logicalFlashSize - 1
		self.configWordAddr = self.logicalFlashSize - 1
		self.osccalAddr = self.flashPageSize - 1
		self.userIDLocationAddr = self.flashPageSize
		self.osccalBackupAddr = self.userIDLocationAddr + self.userIDLocationSize

	def getIHexInterpreter(self):
		inter = IHexInterpreter()
		inter.progmemRanges = [ AddressRange(0, 2 * self.flashPageSize) ]
		inter.fuseRanges = [ AddressRange(2 * self.configWordAddr,
						  2 * self.configWordAddr + 1),
				     AddressRange(2 * 0xFFF,
						  2 * 0xFFF + 1) ]
		inter.uilRanges = [ AddressRange(2 * self.userIDLocationAddr,
						 2 * (self.userIDLocationAddr + self.userIDLocationSize) - 1) ]
		inter.progmemDefaultBytes = self.defaultWord[0] + self.defaultWord[1]
		inter.fuseDefaultBytes = self.defaultWord[0] + self.defaultWord[1]
		return inter

	def setPC(self, address):
		while(self.PC != address):
			self.incrementPC(1)
		
	def incrementPC(self, count):
		for address in range(0, count):
			self.sendCommand(0, 0, 0, self.CMD_INCREMENT_ADDRESS)
			self.PC += 1
			if (self.PC == self.logicalFlashSize):
				self.PC = 0
    
	def sendWriteFlashInstr(self):
		'''
		'''
		# self.loadCommand(self.PROGCMD_SENDDATA)
		# self.top.hostDelay(0.000005)
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_PROGRAMMING)
		self.top.cmdDelay(self.delayTprog)  # 025) #Tprog
		self.sendCommand(0, 0, 0, self.CMD_END_PROGRAMMING)
		self.top.cmdDelay(self.delayTdis)  # Tdis
    
	def sendWriteFlashInstrDM(self):
		self.sendWriteFlashInstr()

	def sendWriteFlashInstrCW(self):
		self.sendWriteFlashInstr()
	def readProgmem(self):
		self.exitPM()
		return Chip_Microchip8_common.readProgmem(self)
	def readFuse(self):
		self.exitPM()
		return Chip_Microchip8_common.readFuse(self)
	def writeFuse(self, image):
		self.exitPM()
		Chip_Microchip8_common.writeFuse(self,image)



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC16F630 DIP14
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_splittedPMarea import *

class Chip_Pic16F630dip14(microchip8_splittedPMarea):
	voltageVDD = 5
	voltageVPP = 9

	userIDLocationSize = 4
	hasEEPROM = True

	CMD_BEGIN_INTERNALLY_TIMED_PROGRAMMING = 0x08
	delayTinternalProgPM = 0.002
	delayTinternalProgDM = 0.005
		
    	def __init__(self):
	    	microchip8_splittedPMarea.__init__(self,
			chipPackage="DIP14",
			chipPinVCC=1,
			chipPinsVPP=4,
			chipPinGND=14,
			signature="\x08\x27",
			flashPageSize=0x400,  # 1024 words
			flashPages=1,
			eepromPageSize=128,
			eepromPages=1,
			fuseBytes=2
			)
		self.configWordAddr = 0x2007
		self.osccalAddr = self.flashPageSize - 1

fuseDesc = (
	BitDescription(0, "FOSC[0], 0=LP, 100=INTOSC"),
	BitDescription(1, "FOSC[1]"),
	BitDescription(2, "FOSC[2]"),
	BitDescription(3, "WDTE, 0=WDT disabled, 1=WDT enabled"),
	BitDescription(4, "nPWRTE"),
	BitDescription(5, "MCLRE"),
	BitDescription(6, "BODEN, 0=BOD disabled"),
	BitDescription(7, "nCP 1=program memory code protection is disabled"),
	BitDescription(8, "nCPD, 1=data memory code protection is disabled"),
	BitDescription(9, "Unused"),
	BitDescription(10, "Unused"),
	BitDescription(11, "Unused"),
	BitDescription(12, "BG[0], Band Gap Calibration bits, 00 = Lowest band gap voltage"),
	BitDescription(13, "BG[1]"),
)

ChipDescription(
	Chip_Pic16F630dip14,
	bitfile="microchip01dip14dip20",
	chipID="pic16F630dip14",
	runtimeID=(0xDE03, 0x01),
	chipVendors="Microchip",
	description="PIC16F630, PIC16F676",
	packages=(("DIP14", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)
